#include "analyzedialog.h"

#include "main.h"
#include "resource.h"

#include "winutils/winutils.h"

AnalyzeDialog::AnalyzeDialog() : Dialog()
{
	g_IsAnalyzing++;
}

AnalyzeDialog::~AnalyzeDialog()
{
	g_IsAnalyzing--;
}

void AnalyzeDialog::AddProject(ProjectPtr project)
{
	for (int gindex = 0; gindex < project->GetGroupCount(); gindex++) {
		GroupPtr group = project->GetGroup(gindex);
		for (int vindex = 0; vindex < group->GetVideoCount(); vindex++) {
			VideoPtr video = group->GetVideo(vindex);
			AddVideo(video);
			for (int aindex = 0; aindex < group->GetAudioCount(); aindex++) {
				AudioPtr audio = video->GetAudio(aindex);
				AddAudio(audio);
			}
			for (int sindex = 0; sindex < group->GetSubtitleCount(); sindex++) {
				SubtitlePtr subtitle = video->GetSubtitle(sindex);
				AddSubtitle(subtitle);
			}
		}
	}
}

void AnalyzeDialog::AddVideo(VideoPtr video)
{
	StuffToAnalyze stuffToAnalyze;
	stuffToAnalyze.m_Video = video;
	m_StuffToAnalyze.push_back(stuffToAnalyze);
	for (int aindex = 0; aindex < video->GetGroup()->GetAudioCount(); aindex++) {
		Audio * audio = video->GetAudio(aindex);
		if (audio && audio->GetFromVideo()) {
			AddAudio(audio);
		}
	}
}

void AnalyzeDialog::AddAudio(AudioPtr audio)
{
	StuffToAnalyze stuffToAnalyze;
	stuffToAnalyze.m_Audio = audio;
	m_StuffToAnalyze.push_back(stuffToAnalyze);
}

void AnalyzeDialog::AddSubtitle(SubtitlePtr subtitle)
{
	StuffToAnalyze stuffToAnalyze;
	stuffToAnalyze.m_Subtitle = subtitle;
	m_StuffToAnalyze.push_back(stuffToAnalyze);
}

bool AnalyzeDialog::DoModal(Dialog * dialog)
{
	return (Dialog::DoModal(hInstance, MAKEINTRESOURCE(IDD_ANALYZE), dialog->hWnd()) != 0);
}

int AnalyzeDialog::GetCount()
{
	return (int) m_StuffToAnalyze.size();
}

VideoPtr AnalyzeDialog::GetVideo(int index)
{
	return m_StuffToAnalyze[index].m_Video;
}

AudioPtr AnalyzeDialog::GetAudio(int index)
{
	return m_StuffToAnalyze[index].m_Audio;
}

SubtitlePtr AnalyzeDialog::GetSubtitle(int index)
{
	return m_StuffToAnalyze[index].m_Subtitle;
}

#define WM_ANALYZE (WM_USER + 1)

bool AnalyzeDialog::DialogProc(UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
	case WM_INITDIALOG:
		PostMessage(hWnd(), WM_ANALYZE, 0, 0);
		return true;
	case WM_ANALYZE:
		EndDialog(hWnd(), DoAnalyze() ? 1 : 0);
		return true;
	}
	return false;
}

bool AnalyzeDialog::DoAnalyze()
{
	bool result = true;
	for (std::vector<StuffToAnalyze>::iterator stuffIterator = m_StuffToAnalyze.begin();
		stuffIterator != m_StuffToAnalyze.end(); stuffIterator++) {
		VideoPtr video = (*stuffIterator).m_Video;
		AudioPtr audio = (*stuffIterator).m_Audio;
		SubtitlePtr subtitle = (*stuffIterator).m_Subtitle;
		if (subtitle()) {
			SetDlgItemText(hWnd(), IDC_FILENAME, subtitle->GetFileName().toFileName().c_str());
			ExecuteMessageLoop();
			if (! subtitle->Analyze()) result = false;
			SetDlgItemText(hWnd(), IDC_FILENAME, TEXT(""));
			ExecuteMessageLoop();
		} else if (audio()) {
			if (! audio->GetFromVideo()) {
				SetDlgItemText(hWnd(), IDC_FILENAME, audio->GetFileName().toFileName().c_str());
				ExecuteMessageLoop();
			}
			if (! audio->Analyze()) result = false;
			if (! audio->GetFromVideo()) {
				SetDlgItemText(hWnd(), IDC_FILENAME, TEXT(""));
				ExecuteMessageLoop();
			}
		} else {
			SetDlgItemText(hWnd(), IDC_FILENAME, video->GetFileName().toFileName().c_str());
			ExecuteMessageLoop();
			if (! video->Analyze()) result = false;
			SetDlgItemText(hWnd(), IDC_FILENAME, TEXT(""));
			ExecuteMessageLoop();
		}
	}
	return result;
}
