#include "aspectpreview.h"
#include "main.h"
#include "resource.h"

AspectPreview::AspectPreview(Dialog * parent, HBITMAP bitmap, int width, int height, int aspectNom, int aspectDen)
{
	m_Parent     = parent;
	m_Bitmap     = bitmap;
	m_OrigWidth  = width;
	m_OrigHeight = height;
	m_AspectNom  = aspectNom;
	m_AspectDen  = aspectDen;
	
	m_DisplayWidth = m_OrigHeight * aspectNom / aspectDen;
	if (m_DisplayWidth >= m_OrigWidth) {
		m_DisplayHeight = m_OrigHeight;
	} else {
		m_DisplayHeight = m_OrigWidth * aspectDen / aspectNom;
		m_DisplayWidth  = m_OrigWidth;
	}
}

AspectPreview::~AspectPreview()
{
}

int AspectPreview::DoModal()
{
	return Dialog::DoModal(hInstance, MAKEINTRESOURCE(IDD_ASPECTPREVIEW), m_Parent->hWnd());
}

bool AspectPreview::DialogProc(UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) 
	{
		case WM_INITDIALOG:
		{
			RECT screenRect;
			SystemParametersInfo(SPI_GETWORKAREA, 0, & screenRect, 0);
			RECT windowRect;
			GetWindowRect(hWnd(), & windowRect);
			RECT clientRect;
			GetClientRect(hWnd(), & clientRect);
			int cx  = m_DisplayWidth  + (windowRect.right - windowRect.left) - (clientRect.right - clientRect.left);
			int cy = m_DisplayHeight + (windowRect.bottom - windowRect.top) - (clientRect.bottom - clientRect.top);
			int x = screenRect.left + (screenRect.right - screenRect.left - cx) / 2;
			int y = screenRect.top  + (screenRect.bottom - screenRect.top - cy) / 2;
			SetWindowPos(hWnd(), NULL, x, y, cx, cy, SWP_NOZORDER);
			ShowWindow(hWnd(), SW_SHOWNORMAL);
			UpdateWindow(hWnd());
			return true;
		}

		case WM_PAINT:
		{
			PAINTSTRUCT ps;
			BeginPaint(hWnd(), & ps);
			HDC dc = GetDC(hWnd());
			HDC compDC = CreateCompatibleDC(dc);
			SelectObject(compDC, m_Bitmap);
			SetStretchBltMode(dc, HALFTONE);
			StretchBlt(dc, 0, 0, m_DisplayWidth, m_DisplayHeight, compDC, 0, 0, m_OrigWidth, m_OrigHeight, SRCCOPY);
			DeleteDC(compDC);
			ReleaseDC(hWnd(), dc);
			EndPaint(hWnd(), & ps);
			return true;
		}

		case WM_LBUTTONDBLCLK:
		case WM_RBUTTONDBLCLK:
		case WM_CLOSE:
		{
			EndDialog(hWnd(), 0);
			return true;
		}

	}
	return false;
}

