#include "csvread.h"

#include <stdlib.h>
#include <string.h>

void csv_free_field(csv_field_t field, int count)
{
	if (field) {
		int i;
		for (i = 0; i < count; i++) { 
			if (field[i].string) {
				free(field[i].string); 
			}
		}
		free(field);
	}
}

void csv_free(csv_file_t csv)
{
	if (csv) {
		if (csv->line) {
			int i;
			for (i = 0; i < csv->count; i++) {
				csv_free_field(csv->line[i].field, csv->line[i].count);
			}
			free(csv->line);
		}
		free(csv);
	}
}

csv_file_t csv_read(FILE * f, int separator, int encloser, int escaper)
{
	csv_file_t csv;
	char * string;
	size_t strsize = 32;
	size_t strlength = 0;
	int fldcount = 0;
	csv_field_t field = NULL;
	int isenclosed  = 0;
	int isescaped   = 0;
	int wasenclosed = 0;
	int c;
	string = (char *) malloc(sizeof(char) * strsize);
	if (! string) { return NULL; }
	csv = (csv_file_t) calloc(1, sizeof(struct csv_file));
	if (! csv) { free(string); return NULL; }
	do {
		int appendtostr = 0;
		int endoffield  = 0;
		int endofline   = 0;
		c = fgetc(f);
		if (c < 0 || 0xFF < c) {
			isenclosed = 0;
			isescaped  = 0;
			endoffield = 1;
			endofline  = 1;			
		} else {
			if (isenclosed && isescaped && (encloser == escaper) && (c != encloser)) {
				isenclosed  = 0;
				isescaped   = 0;
				wasenclosed = 1;
			}
			if (isenclosed) {
				if (isescaped) {
					appendtostr = 1;
					isescaped   = 0;
				} else if (c == escaper) {
					isescaped   = 1;
				} else if (c == encloser) {
					isenclosed  = 0;
					wasenclosed = 1;
				} else {
					appendtostr = 1;
				}
			} else {
				if (c == separator) {
					endoffield  = 1;
				} else if (c == '\n') {
					endoffield  = 1;
					endofline   = 1;
				} else if (c == encloser) {
					isenclosed  = ! wasenclosed;
					if (isenclosed) strlength = 0;
				} else {
					appendtostr = ! wasenclosed;
				}
			}
		}
		if (appendtostr) {
			if (strsize <= strlength) {
				strsize *= 2;
				string = realloc(string, sizeof(char) * strsize);
				if (! string) { csv_free_field(field, fldcount); csv_free(csv); return NULL; }
			}
			string[strlength++] = c;
		}
		if (endoffield) {
			if (fldcount || strlength || wasenclosed || ! endofline) {
				csv_field_t origfield = field;
				field = realloc(field, sizeof(struct csv_field) * (fldcount + 1));
				if (! field) { free(string); csv_free_field(origfield, fldcount); csv_free(csv); return NULL; }
				field[fldcount].string = malloc(sizeof(char) * (strlength + 1));
				if (! field[fldcount].string) { free(string); csv_free_field(field, fldcount); csv_free(csv); return NULL; }
				strncpy(field[fldcount].string, string, strlength);
				field[fldcount].string[strlength] = 0;
				field[fldcount].integer = atoi(field[fldcount].string);
				field[fldcount].real    = atof(field[fldcount].string);
				fldcount++;
			}
			strlength = 0;
			wasenclosed = 0;
		}
		if (endofline && fldcount) {
			csv_line_t origline = csv->line;
			csv->line = realloc(csv->line, sizeof(struct csv_line) * (csv->count + 1));
			if (! csv->line) { free(string); csv_free_field(field, fldcount); csv->line = origline; csv_free(csv); return NULL; }
			csv->line[csv->count].count = fldcount;
			csv->line[csv->count].field = field;
			csv->count++;
			fldcount = 0;
			field = NULL;
		}
	} while (0 <= c && c <= 0xFF);
	free(string);
	return csv;
}

const char * csv_get_str(csv_file_t csv, int lineno, int fieldno, const char * defval)
{
	if (csv && 0 <= lineno && lineno < csv->count && 0 <= fieldno && fieldno < csv->line[lineno].count) {
		return csv->line[lineno].field[fieldno].string;
	} else {
		return defval;
	}
}

int csv_get_int(csv_file_t csv, int lineno, int fieldno, int defval)
{
	if (csv && 0 <= lineno && lineno < csv->count && 0 <= fieldno && fieldno < csv->line[lineno].count) {
		return csv->line[lineno].field[fieldno].integer;
	} else {
		return defval;
	}
}


double csv_get_real(csv_file_t csv, int lineno, int fieldno, double defval)
{
	if (csv && 0 <= lineno && lineno < csv->count && 0 <= fieldno && fieldno < csv->line[lineno].count) {
		return csv->line[lineno].field[fieldno].real;
	} else {
		return defval;
	}
}

