#include "analyzedialog.h"
#include "drawitem.h"
#include "main.h"
#include "maindialog.h"
#include "resource.h"
#include "titledialog.h"
#include "group.h"
#include "video.h"
#include "audio.h"
#include "menudialog.h"
#include "tools.h"
#include "audioencoder.h"
#include "videoencoder.h"
#include "multiplexer.h"
#include "subtitledrawer.h"
#include "menudrawer.h"
#include "dvdauthor.h"
#include "isomaker.h"
#include "dvdburner.h"
#include "avswrapper.h"
#include "configdialog.h"
#include "config.h"

#include "csvread/csvread.h"
#include "winutils/winutils.h"

MainDialog::MainDialog(String projectToOpen, bool checkAviSynthOnly) : Dialog()
{
	m_ProjectToOpen = projectToOpen;
	m_CheckAviSynthOnly = checkAviSynthOnly;

	m_IsCreatingDVD  = false;
	m_IsCancelled    = FALSE;
	m_WaitingForUser = FALSE;

	m_FuncWatchHandle = INVALID_HANDLE_VALUE;
	m_FunctionsChanged = false;
}

MainDialog::~MainDialog()
{
}

void MainDialog::SetProject(ProjectPtr project, String fileName)
{
	m_Project  = project;
	m_FileName = fileName;
}

int MainDialog::DoModal()
{
	return Dialog::DoModal(hInstance, MAKEINTRESOURCE(IDD_MAIN), HWND_DESKTOP);
}

void MainDialog::Update(bool reset)
{
	String dialogTitle;
	if (m_IsCreatingDVD) dialogTitle += S("Creating DVD - ");
	if (m_FileName.length()) dialogTitle += m_FileName.toFileName() + S(" - ");
	dialogTitle += String(APP_VERSION);
	SetWindowText(hWnd(), dialogTitle.c_str());

	EnableWindow(GetDlgItem(hWnd(), IDC_NEW),  (m_FileName.length() || m_Project->HasChanged()) && ! m_IsCreatingDVD);
	EnableWindow(GetDlgItem(hWnd(), IDC_OPEN), ! m_IsCreatingDVD);
	EnableWindow(GetDlgItem(hWnd(), IDC_SAVE), m_Project->HasChanged());
	EnableWindow(GetDlgItem(hWnd(), IDC_CONFIGURE), ! m_IsCreatingDVD);

	EnableWindow(GetDlgItem(hWnd(), IDC_DVDFORMAT_PAL), ! m_IsCreatingDVD);
	SendDlgItemMessage(hWnd(), IDC_DVDFORMAT_PAL, BM_SETCHECK, 
		(m_Project->GetDVDFormat() == DVDFORMAT_PAL ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_DVDFORMAT_NTSC), ! m_IsCreatingDVD);
	SendDlgItemMessage(hWnd(), IDC_DVDFORMAT_NTSC, BM_SETCHECK, 
		(m_Project->GetDVDFormat() == DVDFORMAT_NTSC ? BST_CHECKED : BST_UNCHECKED), 0);

	EnableWindow(GetDlgItem(hWnd(), IDC_DVDSIZE_DVD_5), ! m_IsCreatingDVD);
	SendDlgItemMessage(hWnd(), IDC_DVDSIZE_DVD_5, BM_SETCHECK, 
		(m_Project->GetDVDSize() == DVDSIZE_DVD_5 ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_DVDSIZE_DVD_9), ! m_IsCreatingDVD);
	SendDlgItemMessage(hWnd(), IDC_DVDSIZE_DVD_9, BM_SETCHECK, 
		(m_Project->GetDVDSize() == DVDSIZE_DVD_9 ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_DVDSIZE_CUSTOM), ! m_IsCreatingDVD);
	SendDlgItemMessage(hWnd(), IDC_DVDSIZE_CUSTOM, BM_SETCHECK, 
		(m_Project->GetDVDSize() == DVDSIZE_CUSTOM ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_CUSTOMSIZE), (m_Project->GetDVDSize() == DVDSIZE_CUSTOM) && ! m_IsCreatingDVD);
	SetDlgItemText(hWnd(), IDC_CUSTOMSIZE, String(m_Project->GetCustomSize()).c_str());
	EnableWindow(GetDlgItem(hWnd(), IDC_DVDSIZE_BITRATE), ! m_IsCreatingDVD);
	SendDlgItemMessage(hWnd(), IDC_DVDSIZE_BITRATE, BM_SETCHECK, 
		(m_Project->GetDVDSize() == DVDSIZE_BITRATE ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_CUSTOMBITRATE), (m_Project->GetDVDSize() == DVDSIZE_BITRATE) && ! m_IsCreatingDVD);
	SetDlgItemText(hWnd(), IDC_CUSTOMBITRATE, String(m_Project->GetCustomBitRate()).c_str());

	SetDlgItemText(hWnd(), IDC_STATISTICS, m_Project->GetShortDescr().c_str());

	EnableWindow(GetDlgItem(hWnd(), IDC_OUTPUTTYPE_BURNDVD), ! m_IsCreatingDVD);
	SendDlgItemMessage(hWnd(), IDC_OUTPUTTYPE_BURNDVD, BM_SETCHECK, 
		(m_Project->GetOutputType() == OUTPUTTYPE_BURNDVD ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_OUTPUTTYPE_ISOIMAGE), ! m_IsCreatingDVD);
	SendDlgItemMessage(hWnd(), IDC_OUTPUTTYPE_ISOIMAGE, BM_SETCHECK, 
		(m_Project->GetOutputType() == OUTPUTTYPE_ISOIMAGE ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_OUTPUTTYPE_DVDFOLDER), ! m_IsCreatingDVD);
	SendDlgItemMessage(hWnd(), IDC_OUTPUTTYPE_DVDFOLDER, BM_SETCHECK, 
		(m_Project->GetOutputType() == OUTPUTTYPE_DVDFOLDER ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_OUTPUTTYPE_MPEGFILES), ! m_IsCreatingDVD);
	SendDlgItemMessage(hWnd(), IDC_OUTPUTTYPE_MPEGFILES, BM_SETCHECK, 
		(m_Project->GetOutputType() == OUTPUTTYPE_MPEGFILES ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_OUTPUTTYPE_M2VAC3FILES), ! m_IsCreatingDVD);
	SendDlgItemMessage(hWnd(), IDC_OUTPUTTYPE_M2VAC3FILES, BM_SETCHECK, 
		(m_Project->GetOutputType() == OUTPUTTYPE_M2VAC3FILES ? BST_CHECKED : BST_UNCHECKED), 0);
	
	EnableWindow(GetDlgItem(hWnd(), IDC_OUTPUTDIRECTORY), ! m_IsCreatingDVD);
	SetDlgItemText(hWnd(), IDC_OUTPUTDIRECTORY, m_Project->GetOutputDirectory().c_str());
	EnableWindow(GetDlgItem(hWnd(), IDC_BROWSEOUTPUT), ! m_IsCreatingDVD);

	bool redraw = false;
	SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, WM_SETREDRAW, FALSE, 0);
	if (reset) {
		SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_RESETCONTENT, 0, 0);
		redraw = true;
	}
	int listCount = (int) SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_GETCOUNT, 0, 0);
	int topIndex  = (int) SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_GETTOPINDEX, 0, 0);
	int groupIndex = (int) SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_GETCURSEL, 0, 0);
	bool firstAdded = true;
	for (int index = 0; index < m_Project->GetGroupCount(); index++) {
		if (index < listCount) {
			SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_SETITEMDATA, index, (LPARAM) index);
			MEASUREITEMSTRUCT mis;
			mis.CtlID = IDC_TITLESETLIST;
			mis.itemData = (UINT) index;
			MeasureItem(& mis);
			UINT origHeight = (UINT) SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_GETITEMHEIGHT, index, 0);
			if (mis.itemHeight != origHeight) {
				if (mis.itemHeight < origHeight) redraw = true;
				SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_SETITEMHEIGHT, index, MAKELPARAM(mis.itemHeight, 0));
			}
		} else {
			SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_ADDSTRING, 0, (LPARAM) index);
			if (index >= listCount && firstAdded) {
				topIndex   = index;
				groupIndex = index;
				firstAdded = false;
			}
		}
	}
	for (int index = listCount - 1; index >= m_Project->GetGroupCount(); index--) {
		SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_DELETESTRING, index, 0);
		if (topIndex >= index) topIndex = index - 1;
		if (groupIndex >= index) groupIndex = -1;
		redraw = true;
	}
	SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_SETTOPINDEX, topIndex, 0);
	SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_SETCURSEL, groupIndex, 0);
	SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, WM_SETREDRAW, TRUE, 0);
	InvalidateRect(GetDlgItem(hWnd(), IDC_TITLESETLIST), NULL, redraw);

	EnableWindow(GetDlgItem(hWnd(), IDC_ADDTITLESET),      m_Project->CanAddGroup() && ! m_IsCreatingDVD);
	EnableWindow(GetDlgItem(hWnd(), IDC_MOVETITLESETUP),   m_Project->CanMoveGroupUp(groupIndex) && ! m_IsCreatingDVD);
	EnableWindow(GetDlgItem(hWnd(), IDC_MOVETITLESETDOWN), m_Project->CanMoveGroupDown(groupIndex) && ! m_IsCreatingDVD);
	EnableWindow(GetDlgItem(hWnd(), IDC_DELETETITLESET),   m_Project->CanDeleteGroup(groupIndex) && ! m_IsCreatingDVD);

	if (m_Project->IsGroupIndexValid(groupIndex)) {
		SetDlgItemText(hWnd(), IDC_TITLESETINFO, 
			m_Project->GetGroup(groupIndex)->GetShortDescr(groupIndex).c_str());
	} else {
		SetDlgItemText(hWnd(), IDC_TITLESETINFO, TEXT("Please add or select\na group."));
	}

	Group * group = 
		(m_Project->IsGroupIndexValid(groupIndex)
		? m_Project->GetGroup(groupIndex) : 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_EDITTITLES), group != 0 && ! m_IsCreatingDVD);
	EnableWindow(GetDlgItem(hWnd(), IDC_HASMENU),    group != 0 && ! m_IsCreatingDVD);
	SendDlgItemMessage(hWnd(), IDC_HASMENU, BM_SETCHECK,
		(group && group->HasMenu() ? BST_CHECKED : BST_UNCHECKED), 0);
	if (group && group->HasMenu()) {
		EnableWindow(GetDlgItem(hWnd(), IDC_MENUGROUP), ! m_IsCreatingDVD);
		EnableWindow(GetDlgItem(hWnd(), IDC_EDITROOTMENU), ! m_IsCreatingDVD);
		EnableWindow(GetDlgItem(hWnd(), IDC_NOAUDIOSUBTITLEMENU), ! m_IsCreatingDVD);
		SendDlgItemMessage(hWnd(), IDC_NOAUDIOSUBTITLEMENU, BM_SETCHECK,
			(group->GetMenuType() == MENUTYPE_NONE ? BST_CHECKED : BST_UNCHECKED), 0);
		EnableWindow(GetDlgItem(hWnd(), IDC_COMBINEDAUDIOSUBTITLEMENU), ! m_IsCreatingDVD);
		SendDlgItemMessage(hWnd(), IDC_COMBINEDAUDIOSUBTITLEMENU, BM_SETCHECK,
			(group->GetMenuType() == MENUTYPE_COMBINED ? BST_CHECKED : BST_UNCHECKED), 0);
		EnableWindow(GetDlgItem(hWnd(), IDC_EDITCOMBINEDMENU), (group->GetMenuType() == MENUTYPE_COMBINED) && ! m_IsCreatingDVD);
		EnableWindow(GetDlgItem(hWnd(), IDC_SEPARATEAUDIOSUBTITLEMENU), ! m_IsCreatingDVD);
		SendDlgItemMessage(hWnd(), IDC_SEPARATEAUDIOSUBTITLEMENU, BM_SETCHECK,
			(group->GetMenuType() == MENUTYPE_SEPARATE ? BST_CHECKED : BST_UNCHECKED), 0);
		EnableWindow(GetDlgItem(hWnd(), IDC_HASAUDIOMENU), (group->GetMenuType() == MENUTYPE_SEPARATE) && ! m_IsCreatingDVD);
		SendDlgItemMessage(hWnd(), IDC_HASAUDIOMENU, BM_SETCHECK,
			(group->GetMenuType() == MENUTYPE_SEPARATE && group->HasAudioMenu() ? BST_CHECKED : BST_UNCHECKED), 0);
		EnableWindow(GetDlgItem(hWnd(), IDC_EDITAUDIOMENU), (group->GetMenuType() == MENUTYPE_SEPARATE && group->HasAudioMenu()) && ! m_IsCreatingDVD);
		EnableWindow(GetDlgItem(hWnd(), IDC_HASSUBTITLEMENU), (group->GetMenuType() == MENUTYPE_SEPARATE) && ! m_IsCreatingDVD);
		SendDlgItemMessage(hWnd(), IDC_HASSUBTITLEMENU, BM_SETCHECK,
			(group->GetMenuType() == MENUTYPE_SEPARATE && group->HasSubtitleMenu() ? BST_CHECKED : BST_UNCHECKED), 0);
		EnableWindow(GetDlgItem(hWnd(), IDC_EDITSUBTITLEMENU), (group->GetMenuType() == MENUTYPE_SEPARATE && group->HasSubtitleMenu()) && ! m_IsCreatingDVD);
	} else {
		EnableWindow(GetDlgItem(hWnd(), IDC_MENUGROUP), FALSE);
		EnableWindow(GetDlgItem(hWnd(), IDC_EDITROOTMENU), FALSE);
		EnableWindow(GetDlgItem(hWnd(), IDC_NOAUDIOSUBTITLEMENU), FALSE);
		SendDlgItemMessage(hWnd(), IDC_NOAUDIOSUBTITLEMENU, BM_SETCHECK, BST_UNCHECKED, 0);
		EnableWindow(GetDlgItem(hWnd(), IDC_COMBINEDAUDIOSUBTITLEMENU), FALSE);
		SendDlgItemMessage(hWnd(), IDC_COMBINEDAUDIOSUBTITLEMENU, BM_SETCHECK, BST_UNCHECKED, 0);
		EnableWindow(GetDlgItem(hWnd(), IDC_EDITCOMBINEDMENU), FALSE);
		EnableWindow(GetDlgItem(hWnd(), IDC_SEPARATEAUDIOSUBTITLEMENU), FALSE);
		SendDlgItemMessage(hWnd(), IDC_SEPARATEAUDIOSUBTITLEMENU, BM_SETCHECK, BST_UNCHECKED, 0);
		EnableWindow(GetDlgItem(hWnd(), IDC_HASAUDIOMENU), FALSE);
		SendDlgItemMessage(hWnd(), IDC_HASAUDIOMENU, BM_SETCHECK, BST_UNCHECKED, 0);
		EnableWindow(GetDlgItem(hWnd(), IDC_EDITAUDIOMENU), FALSE);
		EnableWindow(GetDlgItem(hWnd(), IDC_HASSUBTITLEMENU), FALSE);
		SendDlgItemMessage(hWnd(), IDC_HASSUBTITLEMENU, BM_SETCHECK, BST_UNCHECKED, 0);
		EnableWindow(GetDlgItem(hWnd(), IDC_EDITSUBTITLEMENU), FALSE);
	}

	ShowWindow(GetDlgItem(hWnd(), IDC_CREATEDVD), (m_IsCreatingDVD ? SW_HIDE : SW_SHOWNOACTIVATE));
	ShowWindow(GetDlgItem(hWnd(), IDC_CANCELCREATINGDVD), (m_IsCreatingDVD ? SW_SHOWNOACTIVATE : SW_HIDE));
	ShowWindow(GetDlgItem(hWnd(), IDC_SHUTDOWNWHENCOMPLETE), (m_IsCreatingDVD ? SW_SHOWNOACTIVATE : SW_HIDE));

	if ((! m_IsCreatingDVD) && (reset || (m_LastError != m_Project->GetError()))) {
		m_LastError = m_Project->GetError();
		SetDlgItemText(hWnd(), IDC_STATUS, 
			(m_LastError.length() ? m_LastError.crlf().c_str() : TEXT("The project seems to be OK.")));
	}
}

bool MainDialog::DialogProc(UINT message, WPARAM wParam, LPARAM lParam)
{
	switch (message) {
	case WM_INITDIALOG:
		InitDialog();
		return true;
	case WM_COMMAND:
		if (HIWORD(wParam) == BN_CLICKED) {
			switch (LOWORD(wParam)) {
			case IDC_NEW:
				NewProject();
				return true;
			case IDC_OPEN:
				OpenProject();
				return true;
			case IDC_SAVE:
				SaveProject(false);
				return true;
			case IDC_SAVEAS:
				SaveProject(true);
				return true;
			case IDC_CONFIGURE:
				Configure();
				return true;
			case IDC_SHOWHELP:
				Help();
				return true;
			case IDC_SHOWABOUT:
				About();
				return true;
			case IDC_DVDFORMAT_PAL:
				SetDVDFormat(DVDFORMAT_PAL);
				return true;
			case IDC_DVDFORMAT_NTSC:
				SetDVDFormat(DVDFORMAT_NTSC);
				return true;
			case IDC_DVDSIZE_DVD_5:
				m_Project->SetDVDSize(DVDSIZE_DVD_5);
				Update();
				return true;
			case IDC_DVDSIZE_DVD_9:
				m_Project->SetDVDSize(DVDSIZE_DVD_9);
				Update();
				return true;
			case IDC_DVDSIZE_CUSTOM:
				m_Project->SetDVDSize(DVDSIZE_CUSTOM);
				Update();
				return true;
			case IDC_DVDSIZE_BITRATE:
				m_Project->SetDVDSize(DVDSIZE_BITRATE);
				Update();
				return true;
			case IDC_OUTPUTTYPE_BURNDVD:
				if (g_Config->IsImgBurnValid()) {
					m_Project->SetOutputType(OUTPUTTYPE_BURNDVD);
					Update();
				} else {
					ImgBurnMissing();
				}
				return true;
			case IDC_OUTPUTTYPE_ISOIMAGE:
				m_Project->SetOutputType(OUTPUTTYPE_ISOIMAGE);
				Update();
				return true;
			case IDC_OUTPUTTYPE_DVDFOLDER:
				m_Project->SetOutputType(OUTPUTTYPE_DVDFOLDER);
				Update();
				return true;
			case IDC_OUTPUTTYPE_MPEGFILES:
				m_Project->SetOutputType(OUTPUTTYPE_MPEGFILES);
				Update();
				return true;
			case IDC_OUTPUTTYPE_M2VAC3FILES:
				m_Project->SetOutputType(OUTPUTTYPE_M2VAC3FILES);
				Update();
				return true;
			case IDC_BROWSEOUTPUT:
				BrowseOutputDirectory();
				return true;
			case IDC_ADDTITLESET:
				AddGroup();
				return true;
			case IDC_MOVETITLESETUP:
				MoveGroupUp();
				return true;
			case IDC_MOVETITLESETDOWN:
				MoveGroupDown();
				return true;
			case IDC_DELETETITLESET:
				DeleteGroup();
				return true;
			case IDC_EDITTITLES:
				EditTitles();
				return true;
			case IDC_HASMENU:
				SetMenu();
				return true;
			case IDC_NOAUDIOSUBTITLEMENU:
				SetMenuType(MENUTYPE_NONE);
				return true;
			case IDC_COMBINEDAUDIOSUBTITLEMENU:
				SetMenuType(MENUTYPE_COMBINED);
				return true;
			case IDC_SEPARATEAUDIOSUBTITLEMENU:
				SetMenuType(MENUTYPE_SEPARATE);
				return true;
			case IDC_HASAUDIOMENU:
				SetAudioMenu();
				return true;
			case IDC_HASSUBTITLEMENU:
				SetSubtitleMenu();
				return true;
			case IDC_EDITROOTMENU:
				EditMenu(MENU_ROOT);
				return true;
			case IDC_EDITCOMBINEDMENU:
				EditMenu(MENU_COMBINED);
				return true;
			case IDC_EDITAUDIOMENU:
				EditMenu(MENU_AUDIO);
				return true;
			case IDC_EDITSUBTITLEMENU:
				EditMenu(MENU_SUBTITLE);
				return true;
			case IDC_CREATEDVD:
				CreateDVD();
				return true;
			case IDC_SHUTDOWNWHENCOMPLETE:
				if (SendDlgItemMessage(hWnd(), IDC_SHUTDOWNWHENCOMPLETE, BM_GETCHECK, 0, 0) == BST_CHECKED) {
					if (! CanClose(false)) {
						SendDlgItemMessage(hWnd(), IDC_SHUTDOWNWHENCOMPLETE, BM_SETCHECK, BST_UNCHECKED, 0);
					}
				}
				return true;
			case IDC_CANCELCREATINGDVD:
				m_WaitingForUser = TRUE;
				m_IsCancelled = (MessageBox(hWnd(), TEXT("Would you like to cancel creating your DVD?"), APPLICATION, MB_YESNO | MB_ICONQUESTION) == IDYES);
				m_WaitingForUser = FALSE;
				return true;
			default:
				return false;
			}
		} else if (HIWORD(wParam) == EN_KILLFOCUS) {
			switch (LOWORD(wParam)) {
			case IDC_CUSTOMSIZE:
				ChangeCustomSize();
				return true;
			case IDC_CUSTOMBITRATE:
				ChangeCustomBitRate();
				return true;
			case IDC_OUTPUTDIRECTORY:
				ChangeOutputDirectory();
				return true;
			default:
				return false;
			}
		} else if (HIWORD(wParam) == CBN_SELCHANGE) {
			switch (LOWORD(wParam)) {
			case IDC_TITLESETLIST:
				Update();
				return true;
			default: 
				return false;
			}
		}
		return false;

	case WM_MEASUREITEM:
		MeasureItem((LPMEASUREITEMSTRUCT) lParam);
		return true;

	case WM_DRAWITEM:
		DrawItem((LPDRAWITEMSTRUCT) lParam);
		return true;

	case WM_TIMER:
		Timer();
		return true;

	case WM_CLOSE:
	case WM_QUERYENDSESSION:
		if (g_Config->HasChanged()) {
			if (g_Config->Save()) {
				g_Config->ClearChanged();
			}
		}
		if (m_IsCreatingDVD) {
			SendMessage(hWnd(), WM_COMMAND, MAKEWPARAM(IDC_CREATEDVD, BN_CLICKED), 0);
			if (! m_IsCancelled) return false;
		} 
		if (! CanClose()) return false;
		EndDialog(hWnd(), 0);
		return true;

	case WM_DESTROY:
		KillTimer(hWnd(), 1);
		if (m_FuncWatchHandle != INVALID_HANDLE_VALUE) CloseHandle(m_FuncWatchHandle);
		return true;
	}

	return false;
}

void MainDialog::CheckAviSynth()
{
	AVSWrapper avs;
	if (! avs.IsOK()) {
		if (MessageBox(hWnd(), 
			TEXT("It seems that AviSynth is not installed on your computer.")
			TEXT("\n\n")
			TEXT("AviSynth is a video processing utility which is required by DVDforger to convert your videos to DVD format.")
			TEXT("\n\n")
			TEXT("If you'd like to learn more about AviSynth before installing, please visit http://www.avisynth.org")
			TEXT("\n\n")
			TEXT("Would you like to install AviSynth 2.5.7 now?"),
			APPLICATION,
			MB_YESNO | MB_ICONEXCLAMATION) == IDYES) 
		{
			STARTUPINFO si;
			PROCESS_INFORMATION pi;
			ZeroMemory(& si, sizeof(STARTUPINFO));
			si.cb = sizeof(STARTUPINFO);
			if (CreateProcess(tools.AviSynth.c_str(), NULL, NULL, NULL, FALSE, 0, NULL, NULL, & si, & pi)) {
				CloseHandle(pi.hThread);
				WaitForSingleObject(pi.hProcess, INFINITE);
				CloseHandle(pi.hProcess);
			}
		}
	}
}

void MainDialog::InitDialog()
{
	SetClassLong(hWnd(), GCL_HICON, (LONG) LoadIcon(hInstance, MAKEINTRESOURCE(IDI_MAIN)));

	CheckAviSynth();
	if (m_CheckAviSynthOnly) {
		EndDialog(hWnd(), 0);
	} else {

		g_Config = new Config();
		m_FuncWatchHandle = FindFirstChangeNotification(g_Config->GetFunctionDir().c_str(), FALSE, 
			FILE_NOTIFY_CHANGE_FILE_NAME | FILE_NOTIFY_CHANGE_LAST_WRITE);
		SetTimer(hWnd(), 1, 1000, NULL);

		m_Project = new Project();
		m_Project->AddGroup(GroupPtr(new Group(m_Project)), false);
		m_Project->Recalculate();
		Update(true);

		RECT screenRect;
		SystemParametersInfo(SPI_GETWORKAREA, 0, & screenRect, 0);
		int x = (screenRect.right - screenRect.left - 1024) / 2;
		int y = (screenRect.bottom - screenRect.top - 768) / 2;
		RECT windowRect;
		GetWindowRect(hWnd(), & windowRect);
		int cx = windowRect.right - windowRect.left;
		int cy = windowRect.bottom - windowRect.top;
		SetWindowPos(hWnd(), NULL, x, y, cx, cy, SWP_NOZORDER);
		ShowWindow(hWnd(), SW_SHOWDEFAULT);
		UpdateWindow(hWnd());

		if (m_ProjectToOpen.length()) {
			LoadProject(m_ProjectToOpen.toFullPath());
		}
	}
}

void MainDialog::MeasureItem(LPMEASUREITEMSTRUCT mis)
{
	StringVectorVector strings;
	m_Project->GetGroup((int) mis->itemData)->GetDescription(strings);
	MeasureListItem(hWnd(), mis, strings, 0);
}
void MainDialog::DrawItem(LPDRAWITEMSTRUCT dis)
{
	int index = (int) dis->itemData;
	StringVectorVector strings;
	if (m_Project->IsGroupIndexValid(index)) {
		m_Project->GetGroup(index)->GetDescription(strings);
	}
	DrawListItem(hWnd(), dis, strings, dis->itemID + 1);
}

bool MainDialog::CanClose(bool noResult)
{
	bool result = true;
	if (m_Project->HasChanged()) {
		int answer = MessageBox(hWnd(), 
			TEXT("The actual project has unsaved changes. Would you like to save the project now?"),
			APPLICATION,
			MB_YESNOCANCEL |MB_ICONQUESTION);
		switch (answer) {
		case IDYES:
			result = SaveProject(false);
			break;
		case IDNO:
			result = noResult;
			break;
		case IDCANCEL:
			result = false;
			break;
		}
	}
	return result;
}

void MainDialog::NewProject()
{
	if (CanClose()) {
		String openFileDirectory = m_Project->GetOpenFileDirectory();
		m_Project = new Project();
		m_Project->AddGroup(GroupPtr(new Group(m_Project)), false);
		m_Project->SetOpenFileDirectory(openFileDirectory, false);
		m_Project->Recalculate();
		m_FileName.clear();
		Update(true);
	}
}

void MainDialog::OpenProject()
{
	if (CanClose()) {
		TCHAR fileName[MAX_PATH] = TEXT("");
		OPENFILENAME ofn;
		ZeroMemory(&ofn, sizeof(OPENFILENAME));
		ofn.lStructSize     = sizeof(OPENFILENAME);
		ofn.hwndOwner       = hWnd();
		ofn.hInstance       = hInstance;
		ofn.lpstrFilter     = TEXT("DVDforger project\0*.dfp\0All files\0*.*\0\0");
		ofn.lpstrFile       = fileName;
		ofn.nMaxFile        = MAX_PATH;
		ofn.lpstrInitialDir = m_Project->GetOpenFileDirectory().c_str();
		ofn.lpstrTitle      = TEXT("Open project");
		ofn.Flags           = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY;
		ofn.lpstrDefExt     = TEXT("dfp");
		if (GetOpenFileName(&ofn)) {
			LoadProject(String(fileName));
		}
	}
}

void MainDialog::LoadProject(const String & fileName)
{
	bool ok = false;
	FILE * f = fopen(fileName.toAnsi().c_str(), "rt");
	if (f) {
		csv_file_t csv = csv_read(f, ',', '"', '"');
		if (csv) {
			ProjectPtr project = new Project();
			if (project->Load(csv, fileName.replaceFileName(S("")))) {
				if (! project->GetOpenFileDirectory().length()) {
					project->SetOpenFileDirectory(fileName.replaceFileName(S("")), false);
				}
				m_Project  = project;
				m_FileName = fileName;
				ok = true;
			}
			csv_free(csv);
		}
		fclose(f);
	}
	if (ok) {
		Update(true);
		AnalyzeDialog analyzeDialog;
		analyzeDialog.AddProject(m_Project);
		analyzeDialog.DoModal(this);
		m_Project->Recalculate();
		Update();
	} else {
		MessageBox(hWnd(), 
			String(fileName.enclose() + TEXT(" doesn't seem to be a DVDforger project file.")).c_str(),
			APPLICATION, MB_OK | MB_ICONSTOP);
	}
}

bool MainDialog::SaveProject(bool saveAs)
{
	String fileToSave = m_FileName;
	if (saveAs || ! m_FileName.length()) {
		TCHAR fileName[MAX_PATH];
		lstrcpyn(fileName, m_FileName.c_str(), MAX_PATH);
		OPENFILENAME ofn;
		ZeroMemory(&ofn, sizeof(OPENFILENAME));
		ofn.lStructSize     = sizeof(OPENFILENAME);
		ofn.hwndOwner       = hWnd();
		ofn.hInstance       = hInstance;
		ofn.lpstrFilter     = TEXT("DVDforger project\0*.dfp\0All files\0*.*\0\0");
		ofn.lpstrFile       = fileName;
		ofn.nMaxFile        = MAX_PATH;
		ofn.lpstrInitialDir = (m_FileName.length() ? NULL : m_Project->GetOpenFileDirectory().c_str());
		ofn.lpstrTitle      = TEXT("Save project");
		ofn.Flags           = OFN_PATHMUSTEXIST | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT;
		ofn.lpstrDefExt     = TEXT("dfp");
		if (GetSaveFileName(&ofn)) {
			fileToSave = fileName;
		} else {
			return false;
		}
	} 
	
	bool ok = false;
	FILE * f = fopen(fileToSave.toAnsi().c_str(), "wt");
	if (f) {
		ok = true;
		m_Project->Save(f, fileToSave.replaceFileName(S("")));
		if (ferror(f)) ok = false;
		if (fclose(f)) ok = false;
	}
	if (ok) {
		m_Project->ClearChanged();
		m_FileName = fileToSave;
	} else {
		MessageBox(hWnd(), 
				(String(TEXT("Could not save project to file ")) + m_FileName).c_str(),
				APPLICATION, MB_OK | MB_ICONSTOP);
	}
	Update();
	return ok;
}

void MainDialog::Configure()
{
	ConfigDialog(this).DoModal();
}

void MainDialog::Help()
{
	ShellExecute(hWnd(), NULL, tools.help.c_str(), NULL, NULL, SW_SHOWNORMAL);
}

void MainDialog::About()
{
	LPTSTR aboutText = APP_VERSION
		TEXT("\n(c) 2007 Tamas Kurucsai")
		TEXT("\nLicensed under the terms of the GNU General Public License.")
		TEXT("\nHomepage: http://dvdforger.googlepages.com/.")
		TEXT("\n")
		TEXT("\nThe package contains the following additional software:")
		TEXT("\n- Aften 0.07 - by Justin Ruggles, Prakash Punnoor, et al.")
		TEXT("\n- AviSynth 2.5.7 installer")
		TEXT("\n- bmp2png 1.62 - by Miyasaka Masaru")
		TEXT("\n- Decomb 5.2.2 - by Donald A. Graft")
		TEXT("\n- dvdauthor 0.6.14 - by Scott Smith")
		TEXT("\n- HC Encoder 0.21 - by hank315")
		TEXT("\n- mkisofs 2.01 - by Joerg Schilling")
		TEXT("\n- mplex 2.2.7 from mjpegtools 1.9.0rc2")
		TEXT("\n- QuEnc 0.72 - by Nic & DZG")
		TEXT("\nSee copyright notices and licenses in the 'tools' directory.")
		TEXT("\n")
		TEXT("\nThe source code contains the library EasyBMP 1.06 by Paul Macklin.")
		TEXT("\nSee copyright notice and license in the 'source' directory.")
		TEXT("\n")
		TEXT("\nThe eye icon is from http://www.geocities.com/kweviltiger606/downloads.html.");
	MSGBOXPARAMS mbp;
	ZeroMemory(& mbp, sizeof(MSGBOXPARAMS));
	mbp.cbSize      = sizeof(MSGBOXPARAMS);
	mbp.hwndOwner   = hWnd();
	mbp.hInstance   = hInstance;
	mbp.lpszText    = aboutText;
	mbp.lpszCaption = APPLICATION;
	mbp.dwStyle     = MB_USERICON | MB_OK;
	mbp.lpszIcon    = MAKEINTRESOURCE(IDI_MAIN);
	MessageBoxIndirect(& mbp);
}

void MainDialog::SetDVDFormat(int dvdFormat)
{
	if (dvdFormat != m_Project->GetDVDFormat()) {
		m_Project->SetDVDFormat(dvdFormat);
		AnalyzeDialog analyzeDialog;
		analyzeDialog.AddProject(m_Project);
		if (analyzeDialog.GetCount()) analyzeDialog.DoModal(this);
		Update();
	}
}

void MainDialog::ChangeCustomSize()
{
	int customSize = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_CUSTOMSIZE)).toAnsi().c_str());
	if (customSize > 0) m_Project->SetCustomSize(customSize);
	Update();
}

void MainDialog::ChangeCustomBitRate()
{
	int customBitRate = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_CUSTOMBITRATE)).toAnsi().c_str());
	if (customBitRate > 0) m_Project->SetCustomBitRate(customBitRate);
	Update();
}

void MainDialog::ChangeOutputDirectory()
{
	m_Project->SetOutputDirectory(String().fromWindow(GetDlgItem(hWnd(), IDC_OUTPUTDIRECTORY)).toFullPath());
	Update();
}

void MainDialog::BrowseOutputDirectory()
{
	TCHAR outputDirectory[MAX_PATH];
	if (m_Project->GetOutputDirectory().length()) {
		lstrcpyn(outputDirectory, m_Project->GetOutputDirectory().c_str(), MAX_PATH);
	} else {
		lstrcpyn(outputDirectory, m_Project->GetOpenFileDirectory().c_str(), MAX_PATH);
	}
	if (BrowseForDirectory(hWnd(), outputDirectory, TEXT("Output directory:"), TRUE)) {
		m_Project->SetOutputDirectory(String(outputDirectory));
		Update();
	}
}

void MainDialog::AddGroup()
{
	if (m_Project->AddGroup(GroupPtr(new Group(m_Project)))) {
		Update();
	}
}

void MainDialog::MoveGroupUp()
{
	int groupIndex = (int) SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_GETCURSEL, 0, 0);
	if (m_Project->MoveGroupUp(groupIndex)) {
		SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_SETCURSEL, --groupIndex, 0);
		Update();
	}
}

void MainDialog::MoveGroupDown()
{
	int groupIndex = (int) SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_GETCURSEL, 0, 0);
	if (m_Project->MoveGroupDown(groupIndex)) {
		SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_SETCURSEL, ++groupIndex, 0);
		Update();
	}
}

void MainDialog::DeleteGroup()
{
	int groupIndex = (int) SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_GETCURSEL, 0, 0);
	if (m_Project->DeleteGroup(groupIndex)) {
		SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_SETCURSEL, (WPARAM) -1, 0);
		Update();
	}
}

void MainDialog::EditTitles()
{
	int groupIndex = (int) SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_GETCURSEL, 0, 0);
	if (m_Project->IsGroupIndexValid(groupIndex)) {
		TitleDialog titleDialog(this, m_Project->GetGroup(groupIndex), groupIndex);
		titleDialog.DoModal();
	}
}

void MainDialog::SetMenu()
{
	int groupIndex = (int) SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_GETCURSEL, 0, 0);
	if (m_Project->IsGroupIndexValid(groupIndex)) {
		m_Project->GetGroup(groupIndex)->SetMenu(! m_Project->GetGroup(groupIndex)->HasMenu());
		Update();
	}
}

void MainDialog::SetMenuType(int type)
{
	int groupIndex = (int) SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_GETCURSEL, 0, 0);
	if (m_Project->IsGroupIndexValid(groupIndex)) {
		m_Project->GetGroup(groupIndex)->SetMenuType(type);
		Update();
	}
}

void MainDialog::SetAudioMenu()
{
	int groupIndex = (int) SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_GETCURSEL, 0, 0);
	if (m_Project->IsGroupIndexValid(groupIndex)) {
		m_Project->GetGroup(groupIndex)->SetAudioMenu(! m_Project->GetGroup(groupIndex)->HasAudioMenu());
		Update();
	}
}

void MainDialog::SetSubtitleMenu()
{
	int groupIndex = (int) SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_GETCURSEL, 0, 0);
	if (m_Project->IsGroupIndexValid(groupIndex)) {
		m_Project->GetGroup(groupIndex)->SetSubtitleMenu(! m_Project->GetGroup(groupIndex)->HasSubtitleMenu());
		Update();
	}
}

void MainDialog::EditMenu(int which)
{
	int groupIndex = (int) SendDlgItemMessage(hWnd(), IDC_TITLESETLIST, LB_GETCURSEL, 0, 0);
	if (m_Project->IsGroupIndexValid(groupIndex)) {
		MenuPtr menu;
		switch (which) {
		case MENU_ROOT:
			menu = m_Project->GetGroup(groupIndex)->GetMenu();
			break;
		case MENU_COMBINED:
			menu = m_Project->GetGroup(groupIndex)->GetCombinedMenu();
			break;
		case MENU_AUDIO:
			menu = m_Project->GetGroup(groupIndex)->GetAudioMenu();
			break;
		case MENU_SUBTITLE:
			menu = m_Project->GetGroup(groupIndex)->GetSubtitleMenu();
			break;
		}
		MenuDialog menuDialog(this, menu, groupIndex);
		menuDialog.DoModal();
	}
}

void MainDialog::ImgBurnMissing()
{
	MessageBox(hWnd(), 
		TEXT("DVDforger requires ImgBurn to be installed to burn DVDs. However, it seems that ImgBurn is not installed.\n\n")
		TEXT("If you have installed ImgBurn somewhere else than its default installation location, please specify this location using 'Configuration' above.\n\n")
		TEXT("ImgBurn is a freeware application. It can be downloaded from http://www.imgburn.com/.\n"),
		APPLICATION, MB_OK | MB_ICONEXCLAMATION);
}

void MainDialog::AppendStatus(const String & text)
{
	int length = GetWindowTextLength(GetDlgItem(hWnd(), IDC_STATUS));
	SendDlgItemMessage(hWnd(), IDC_STATUS, EM_SETSEL, length, length);
	SendDlgItemMessage(hWnd(), IDC_STATUS, EM_REPLACESEL, FALSE, (LPARAM) text.c_str());
	length = GetWindowTextLength(GetDlgItem(hWnd(), IDC_STATUS));
	SendDlgItemMessage(hWnd(), IDC_STATUS, EM_SETSEL, length, length);
	SendDlgItemMessage(hWnd(), IDC_STATUS, EM_SCROLLCARET, 0, 0);
}

void MainDialog::CreateDVD()
{
	if (m_Project->GetError().length()) {
		MessageBox(hWnd(), TEXT("Please correct the errors listed under 'Project status' first."), APPLICATION, MB_ICONEXCLAMATION | MB_OK);
	} else if ((m_Project->GetOutputType() == OUTPUTTYPE_BURNDVD) && ! g_Config->IsImgBurnValid()) {
		ImgBurnMissing();
	} else {
		int spaceNeeded = (m_Project->GetDVDCapacity() * 2) / 1024 + 2;
		if (MessageBox(hWnd(), (S("You are about to create a DVD from your video files.\n\n")
			+ S("Warning: all files in all subdirectories inside ") + m_Project->GetOutputDirectory().enclose() + S(" will be deleted!\n\n")
			+ S("Please also make sure that you have at least ") + String(spaceNeeded) + S(" gigabytes of free space there.\n\n")
			+ S("Would you like create the DVD now?")).c_str(), APPLICATION, MB_YESNO | MB_ICONQUESTION) == IDYES) {
			if (! CreateDirectoryTree(m_Project->GetOutputDirectory().c_str(), FALSE, NULL)) {
				MessageBox(hWnd(), 
					(S("Could not create directory ") + m_Project->GetOutputDirectory().enclose()).c_str(),
					APPLICATION, MB_OK | MB_ICONSTOP);
			} else if (! IsDirectoryEmpty(m_Project->GetOutputDirectory().c_str()) &&
				MessageBox(hWnd(), (S("The directory ") + m_Project->GetOutputDirectory() + S(" is not empty!\n\n")
					+ S("Are you really sure you want to delete everything inside it?")).c_str(), APPLICATION,
					MB_YESNO | MB_ICONEXCLAMATION) != IDYES) {
				// nothing to do
			#ifndef VSTUDIO_BUILD
			} else if (! CleanDirectory(m_Project->GetOutputDirectory().c_str())) {
				MessageBox(hWnd(), 
					(S("Could not empty directory ") + m_Project->GetOutputDirectory().enclose()).c_str(),
					APPLICATION, MB_OK | MB_ICONSTOP);
			#endif
			} else {
				m_IsCreatingDVD  = true;
				m_IsCancelled    = FALSE;
				m_WaitingForUser = FALSE;
				Update();
				String origCaption = String().fromWindow(GetDlgItem(hWnd(), IDC_CREATEDVD));
				SetWindowText(GetDlgItem(hWnd(), IDC_CREATEDVD), TEXT("Cancel creating DVD"));
				SetDlgItemText(hWnd(), IDC_STATUS, TEXT(""));
				ThreadVector threads;
				for (int gindex = 0; gindex < m_Project->GetGroupCount(); gindex++) {
					GroupPtr group = m_Project->GetGroup(gindex);
					for (int tindex = 0; tindex < group->GetTitleCount(); tindex++) {
						for (int aindex = 0; aindex < group->GetAudioCount(); aindex++) {
							threads.push_back(ThreadPtr(new AudioEncoder(m_Project, gindex, tindex, aindex)));
						}
						threads.push_back(ThreadPtr(new VideoEncoder(m_Project, gindex, tindex)));
						if (m_Project->GetOutputType() <= OUTPUTTYPE_MPEGFILES) {
							threads.push_back(ThreadPtr(new Multiplexer(m_Project, gindex, tindex)));
							for (int sindex = 0; sindex < group->GetSubtitleCount(); sindex++) {
								threads.push_back(ThreadPtr(new SubtitleDrawer(m_Project, gindex, tindex, sindex)));
							}
						}
					}
					if (m_Project->GetOutputType() <= OUTPUTTYPE_DVDFOLDER) {
						if (group->HasMenu()) {
							threads.push_back(ThreadPtr(new MenuDrawer(m_Project, gindex, MENU_ROOT)));
						}
						if (group->GetMenuType() == MENUTYPE_COMBINED) {
							threads.push_back(ThreadPtr(new MenuDrawer(m_Project, gindex, MENU_COMBINED)));
						} else if (group->GetMenuType() == MENUTYPE_SEPARATE) {
							if (group->HasAudioMenu()) 
								threads.push_back(ThreadPtr(new MenuDrawer(m_Project, gindex, MENU_AUDIO)));
							if (group->HasSubtitleMenu()) 
								threads.push_back(ThreadPtr(new MenuDrawer(m_Project, gindex, MENU_SUBTITLE)));
						}
					}
				}
				if (m_Project->GetOutputType() <= OUTPUTTYPE_DVDFOLDER) {
					threads.push_back(ThreadPtr(new DVDAuthor(m_Project)));
				}
				if (m_Project->GetOutputType() <= OUTPUTTYPE_ISOIMAGE) {
					threads.push_back(ThreadPtr(new ISOMaker(m_Project)));
				}
				if (m_Project->GetOutputType() <= OUTPUTTYPE_BURNDVD) {
					threads.push_back(ThreadPtr(new DVDBurner(m_Project)));
				}
				bool ok = true;
				int step = 0;
				for (ThreadVector::iterator ti = threads.begin(); ok && ti != threads.end(); ti++) {
					AppendStatus(S("Step ") + String(++step) + S(" of ") + String((int) threads.size()) + S(": ")
						+ (*ti)->GetDescription() + S("..."));
					ExecuteMessageLoop();
					ok = (*ti)->Run(& m_IsCancelled, & m_WaitingForUser);
					if (m_IsCancelled) {
						AppendStatus(S(" cancelled."));
						ok = false;
					} else {
						AppendStatus(ok ? S(" done.\r\n") : S(" Error!\r\n"));
						ExecuteMessageLoop();
						if (! ok) {
							if (MessageBox(hWnd(), TEXT("An error occured while creating your DVD. Sorry.\nWould you like to see the log?"), APPLICATION, MB_YESNO | MB_ICONSTOP) == IDYES) {
								String logFile = String(m_Project->GetOutputDirectory() + S("\\log.txt")).toFullPath();
								ShellExecute(hWnd(), NULL, logFile.c_str(), NULL, NULL, SW_SHOWNORMAL);
							}
						}
					}
				}
				m_IsCreatingDVD  = false;
				m_IsCancelled    = FALSE;
				m_WaitingForUser = FALSE;
				SetWindowText(GetDlgItem(hWnd(), IDC_CREATEDVD), origCaption.c_str());
				Update();
				if (ok) {
					#ifndef VSTUDIO_BUILD
					String logFileName = String(m_Project->GetOutputDirectory() + S("\\") + S("log.txt")).toFullPath();
					DeleteFile(logFileName.c_str());
					#endif
					AppendStatus(S("Your DVD is ready."));
					if (SendDlgItemMessage(hWnd(), IDC_SHUTDOWNWHENCOMPLETE, BM_GETCHECK, 0, 0) == BST_CHECKED) {
						OSVERSIONINFO info; 
						info.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
						GetVersionEx(& info);
						if (info.dwPlatformId == VER_PLATFORM_WIN32_NT) {
							HANDLE process = GetCurrentProcess();
							HANDLE token;
							if (OpenProcessToken(process, TOKEN_QUERY | TOKEN_ADJUST_PRIVILEGES, & token)) {
								HLOCAL memhdl;
								TOKEN_PRIVILEGES * privileges;
								LUID_AND_ATTRIBUTES * privilege;
								memhdl = LocalAlloc(0, sizeof(TOKEN_PRIVILEGES) + sizeof(LUID_AND_ATTRIBUTES)); 
								privileges = (TOKEN_PRIVILEGES *) LocalLock(memhdl);
								privileges->PrivilegeCount = 1; 
								privilege = (LUID_AND_ATTRIBUTES *) & (privileges->Privileges); 
								LookupPrivilegeValue(TEXT(""), SE_SHUTDOWN_NAME, & privilege->Luid);
								privilege->Attributes = SE_PRIVILEGE_ENABLED;
								AdjustTokenPrivileges(token, FALSE, privileges, 0, NULL, 0);
							}
						}
						ExitWindowsEx(EWX_POWEROFF, 0);
					} else {
						MessageBox(hWnd(), TEXT("Your DVD is ready."), APPLICATION, MB_ICONINFORMATION | MB_OK);
					}
				}
			}
		}
	}
}

void MainDialog::Timer()
{
	if (m_FuncWatchHandle != INVALID_HANDLE_VALUE) {
		if (WaitForSingleObject(m_FuncWatchHandle, 0) == WAIT_OBJECT_0) {
			m_FunctionsChanged = true;
			FindNextChangeNotification(m_FuncWatchHandle);
		} else if (m_FunctionsChanged && ! m_IsCreatingDVD && ! g_IsAnalyzing) {
			m_FunctionsChanged = false;
			if (g_Config->RefreshFunctionMaps()) {
				AnalyzeDialog analyzeDialog;
				analyzeDialog.AddProject(m_Project);
				if (analyzeDialog.GetCount()) {
					analyzeDialog.DoModal(Dialog::GetActDialog());
				}
				m_Project->Recalculate();
				Dialog::GetActDialog()->Refresh();
			}
		}
	}
}