#ifndef MENU_H_
#define MENU_H_

#include "managableptr.h"
#include "managedptr.h"
#include "string.h"
#include "menuitem.h"

#include <vector>

#include "csvread/csvread.h"

class Group;

#define MENU_ROOT     0
#define MENU_COMBINED 1
#define MENU_AUDIO    2
#define MENU_SUBTITLE 3

#define MENUCOLOR_BACKGROUND 0
#define MENUCOLOR_NORMAL     1
#define MENUCOLOR_SELECT     2
#define MENUCOLOR_HILITE     3

#define ROOTMENU_PLAYTITLE    0
#define ROOTMENU_GOTOCOMBINED 1
#define ROOTMENU_GOTOAUDIO    2
#define ROOTMENU_GOTOSUBTITLE 3
#define ROOTMENU_GOTOTITLESET 4
#define ROOTMENU_RESUME       5
#define ROOTMENU_MAX          6

#define MAX_MENU_ITEMS 31

class Menu : public ManagablePtr
{
public:
	Menu(Group * group, int type);
	virtual ~Menu();
	Group * GetGroup() const;
	int GetType() const;

public:
	const String & GetBackgroundImage() const;
	bool SetBackgroundImage(const String & backgroundImage, bool setChanged = true);
	COLORREF GetBackgroundColor() const;
	bool SetBackgroundColor(COLORREF backgroundColor, bool setChanged = true);
	COLORREF GetNormalColor() const;
	bool SetNormalColor(COLORREF normalColor, bool setChanged = true);
	COLORREF GetSelectColor() const;
	bool SetSelectColor(COLORREF selectColor, bool setChanged = true);
	COLORREF GetHiliteColor() const;
	bool SetHiliteColor(COLORREF hiliteColor, bool setChanged = true);
	COLORREF GetColor(int which) const;
	bool GetAutoResume() const;
	bool SetAutoResume(bool autoResume, bool setChanged = true);

	bool CanAddMenuItem() const;
	bool AddMenuItem(MenuItemPtr & menuItem, bool setChanged = true);
	bool IsMenuItemIndexValid(int index) const;
	int GetMenuItemCount() const;
	MenuItemPtr GetMenuItem(int index) const;
	bool SelectMenuItem(int index, bool selected);
	bool ClearSelection();
	int GetSelectedCount() const;
	int GetFirstSelected() const;
	MenuItemPtr GetSelectedMenuItem() const;
	bool IsMenuItemSelected(int index) const;
	bool CanDeleteSelectedMenuItems() const;
	bool DeleteSelectedMenuItems(bool setChanged = true);

	void SetChanged();
	String GetIdent();
	String Recalculate();

	bool Load(csv_file_t csv, int & lineno, int nest);
	void Save(FILE * f, int nest) const;

protected:
	Group * m_Group;
	int m_Type;
	String m_BackgroundImage;
	COLORREF m_BackgroundColor;
	COLORREF m_NormalColor;
	COLORREF m_SelectColor;
	COLORREF m_HiliteColor;
	bool m_AutoResume;
	MenuItemVector m_MenuItems;
	int m_SelectedCount;
	int m_FirstSelected;

public:
	int authorIndex;
};

typedef ManagedPtr<Menu> MenuPtr;
typedef std::vector<MenuPtr> MenuVector;

#endif // MENU_H_
