#ifndef MENUITEM_H_
#define MENUITEM_H_

#include "managableptr.h"
#include "managedptr.h"
#include "string.h"

#include <vector>

#include "csvread/csvread.h"

#include <windows.h>

class Menu;

#define ALIGN_TOPLEFT      0
#define ALIGN_TOPCENTER    1
#define ALIGN_TOPRIGHT     2
#define ALIGN_CENTERLEFT   3
#define ALIGN_CENTERCENTER 4
#define ALIGN_CENTERRIGHT  5
#define ALIGN_BOTTOMLEFT   6
#define ALIGN_BOTTOMCENTER 7
#define ALIGN_BOTTOMRIGHT  8
#define ALIGN_MAX          9

#define DRAG_SELECT      -1
#define DRAG_TOPLEFT     0
#define DRAG_TOP         1
#define DRAG_TOPRIGHT    2
#define DRAG_LEFT        3
#define DRAG_RIGHT       4
#define DRAG_BOTTOMLEFT  5 
#define DRAG_BOTTOM      6
#define DRAG_BOTTOMRIGHT 7
#define DRAG_CORNERS     8
#define DRAG_MOVE        9
#define DRAG_MAX         10

class MenuItem : public ManagablePtr
{
public:
	MenuItem(Menu * menu, int selector1 = 0, int selector2 = 0);
	virtual ~MenuItem();
	Menu * GetMenu() const;

public:
	bool IsSelected() const;
	void SetSelected(bool selected);

	int GetSelector1() const;
	bool SetSelector1(int selector1, bool setChanged = true);
	int GetSelector2() const;
	bool SetSelector2(int selector2, bool setChanged = true);
	int GetX() const;
	bool SetX(int x, bool setChanged = true);
	int GetY() const;
	bool SetY(int y, bool setChanged = true);
	bool GetNormalText() const;
	bool SetNormalText(bool normalText, bool setChanged = true);
	bool GetHiliteText() const;
	bool SetHiliteText(bool hiliteText, bool setChanged = true);
	const String & GetCaption() const;
	bool SetCaption(const String & caption, bool setChanged = true);
	const String & GetFontFace();
	bool SetFontFace(const String & fontFace, bool setChanged = true);
	int GetFontSize() const;
	bool SetFontSize(int fontSize, bool setChanged = true);
	bool GetBold() const;
	bool SetBold(bool bold, bool setChanged = true);
	bool GetItalic() const;
	bool SetItalic(bool italic, bool setChanged = true);
	bool GetRectangle() const;
	bool GetNormalRect() const;
	bool SetNormalRect(bool normalRect, bool setChanged = true);
	bool GetHiliteRect() const;
	bool SetHiliteRect(bool hiliteRect, bool setChanged = true);
	int GetRight() const;
	bool SetRight(int right, bool setChanged = true);
	int GetBottom() const;
	bool SetBottom(int bottom, bool setChanged = true);
	int GetThickness() const;
	bool SetThickness(int thickness, bool setChanged = true);
	int GetHRadius() const;
	bool SetHRadius(int hRadius, bool setChanged = true);
	int GetVRadius() const;
	bool SetVRadius(int hRadius, bool setChanged = true);
	bool GetFillNormal() const;
	bool SetFillNormal(bool fillNormal, bool setChanged = true);
	bool GetFillHilite() const;
	bool SetFillHilite(bool fillHilite, bool setChanged = true);
	bool GetAutoResize() const;
	bool SetAutoResize(bool autoResize, bool setChanged = true);
	int GetHMargin() const;
	bool SetHMargin(int hMargin, bool setChanged = true);
	int GetVMargin() const;
	bool SetVMargin(int hMargin, bool setChanged = true);
	int GetAlignment() const;
	bool SetAlignment(int alignment, bool setChanged = true);

	void SetChanged();
	int GetIndex() const;
	void SetIndex(int index);
	String GetIdent();
	String Recalculate();
	bool IsValid() const;

	const RECT & GetBaseRect() const;
	const RECT & GetTextRect() const;
	const RECT & GetDrawnRect() const;
	const RECT & GetAdjustedRect() const;
	const RECT & GetDragRect(int index) const;
	HFONT CreateFont() const;

	void GetDescription(StringVectorVector & strings) const;
	String GetShortDescr() const;

	bool operator < (const MenuItem & menuItem) const;

	bool Load(csv_file_t csv, int & lineno, int nest);
	void Save(FILE * f, int nest) const;

protected:
	Menu * m_Menu;
	bool m_IsSelected;
	int m_Selector1;
	int m_Selector2;
	int m_X;
	int m_Y;
	bool m_NormalText;
	bool m_HiliteText;
	String m_Caption;
	String m_FontFace;
	int m_FontSize;
	bool m_Bold;
	bool m_Italic;
	bool m_NormalRect;
	bool m_HiliteRect;
	int  m_Right;
	int  m_Bottom;
	int  m_Thickness;
	int  m_HRadius;
	int  m_VRadius;
	bool m_FillNormal;
	bool m_FillHilite;
	bool m_AutoResize;
	int  m_HMargin;
	int  m_VMargin;
	int  m_Alignment;
	bool m_IsValid;
	RECT m_BaseRect;
	RECT m_TextRect;
	RECT m_DrawnRect;
	RECT m_AdjustedRect;
	RECT m_DragRect[DRAG_CORNERS];
	int m_Index;

public:
	int authorIndex;
};

/*
class MenuItemPtr : public ManagedPtr<MenuItem>
{
public:
	MenuItemPtr() : ManagedPtr<MenuItem>() {};
	MenuItemPtr(MenuItem * mi) : ManagedPtr<MenuItem>(mi) {};
	MenuItemPtr(const MenuItemPtr & mi) : ManagedPtr<MenuItem>(mi) {};
	virtual ~MenuItemPtr() {};
	// now comes the stuff why typedef wasn't enough:
	bool operator < (const MenuItemPtr & mi) { return ManagedPtr<MenuItem>::operator < (mi); };
	// in such cases, I really recommend the C++ section of the Unix Haters' Handbook
};
*/
typedef ManagedPtr<MenuItem> MenuItemPtr;
typedef std::vector<MenuItemPtr> MenuItemVector;

#endif // MENUITEM_H_

