#include "multiplexer.h"
#include "group.h"
#include "tools.h"

#include <stdio.h>
#include "safefgets/safefgets.h"
#include "winutils/winutils.h"

Multiplexer::Multiplexer(ProjectPtr project, int groupIndex, int titleIndex) : Thread()
{
	m_Project = project;
	m_GroupIndex = groupIndex;
	m_TitleIndex = titleIndex;
}

Multiplexer::~Multiplexer()
{
}

String Multiplexer::GetDescription() const
{
	return S("Multiplexing audio and video of title ") + String(m_TitleIndex + 1) + S(" in group ") + String(m_GroupIndex + 1);
}

bool Multiplexer::OnRun()
{
	bool ok = true;
	
	String baseFileName = String(m_Project->GetOutputDirectory() + S("\\set") +
		String(m_GroupIndex + 1) + S("title") + String(m_TitleIndex + 1)).toFullPath();
	String mplexCommand = S("-f 8 -o ") + String(baseFileName + S(".mpg")).enclose();
	String mpgFileName = baseFileName + S("video.m2v");
	mplexCommand += (S(" ") + mpgFileName.enclose());
	AddFileToDelete(mpgFileName);
	for (int index = 0; index < m_Project->GetGroup(m_GroupIndex)->GetAudioCount(); index++) {
		String ac3FileName = baseFileName + S("audio") + String(index + 1) + S(".ac3");
		mplexCommand += String(S(" ") + ac3FileName.enclose());
		AddFileToDelete(ac3FileName);
	}
	String logFileName = String(m_Project->GetOutputDirectory() + S("\\") + S("log.txt")).toFullPath();

	ok = ok && ExecuteCommand(tools.mplex.c_str(), mplexCommand.c_str(), 
		NULL, NULL, logFileName.c_str(), isCancelled, waitingForUser);

	if (! ok) {
		ok = IsFakeError(logFileName);
	}

	if (ok) DeleteFilesNow();
	return ok;
}

bool Multiplexer::IsFakeError(const String & logFileName)
{
	bool headFound = false;
	int fakeErrors = 0;
	int realErrors = 0;
	FILE * f = fopen(logFileName.toAnsi().c_str(), "rt");
	if (f) {
		char * line = NULL;
		while ((line = safefgets(line, f)) != NULL) {
			int x;
			if (sscanf(line, "   INFO: [???] mplex version %d", &x) == 1) {
				headFound = true;
				fakeErrors = 0;
				realErrors = 0;
			} else if (headFound) {
				if (! strcmp(line, "**ERROR: [???] MUX STATUS: Frame data under-runs detected!\n")) {
					fakeErrors++;
				} else if (! strncmp(line, "**ERROR: [???]", 14)) {
					realErrors++;
				}
			}
		}
		fclose(f);
	}
	return (headFound && (fakeErrors == 1) && (realErrors == 0));
}
