#ifndef PROJECTOBJ_H_
#define PROJECTOBJ_H_

#include "managableptr.h"
#include "managedptr.h"
#include "string.h"
#include "csvread/csvread.h"

#include "group.h"


#define DVDFORMAT_PAL  0
#define DVDFORMAT_NTSC 1
#define DVDFORMAT_MAX  2

#define DVDSIZE_DVD_5   0
#define DVDSIZE_DVD_9   1
#define DVDSIZE_CUSTOM  2
#define DVDSIZE_BITRATE 3
#define DVDSIZE_MAX     4

#define OUTPUTTYPE_BURNDVD     0
#define OUTPUTTYPE_ISOIMAGE    1
#define OUTPUTTYPE_DVDFOLDER   2
#define OUTPUTTYPE_MPEGFILES   3
#define OUTPUTTYPE_M2VAC3FILES 4
#define OUTPUTTYPE_MAX         5

#define MAX_GROUP_COUNT 5

class Project : public ManagablePtr
{
public:
	Project();
	virtual ~Project();

public:
	bool SetDVDFormat(int dvdFormat, bool setChanged = true);
	int GetDVDFormat() const;
	bool SetDVDSize(int dvdSize, bool setChanged = true);
	int GetDVDSize() const;
	bool SetCustomSize(int customSize, bool setChanged = true);
	int GetCustomSize() const;
	bool SetCustomBitRate(int customBitRate, bool setChanged = true);
	int GetCustomBitRate() const;
	bool SetOutputType(int outputType, bool setChanged = true);
	int GetOutputType() const;
	bool SetOutputDirectory(const String & outputDirectory, bool setChanged = true);
	const String & GetOutputDirectory() const;
	bool SetOpenFileDirectory(const String & openFileDirectory, bool setChanged = true);
	const String & GetOpenFileDirectory() const;

	int GetGroupCount() const;
	bool IsGroupIndexValid(int index) const;
	GroupPtr GetGroup(int index) const;
	bool CanAddGroup() const;
	bool AddGroup(GroupPtr group, bool setChanged = true);
	bool CanMoveGroupUp(int index) const;
	bool MoveGroupUp(int index, bool setChanged = true);
	bool CanMoveGroupDown(int index) const;
	bool MoveGroupDown(int index, bool setChanged = true);
	bool CanDeleteGroup(int index) const;
	bool DeleteGroup(int index, bool setChanged = true);

	bool HasChanged() const;
	void SetChanged();
	void ClearChanged();
	void Recalculate();
	const String & GetError() const;
	
public:
	String GetVideoFormat() const;
	int GetVideoWidth() const;
	int GetVideoHeight() const;
	double GetVideoWHRatio() const;
	double GetAspectCorr(int aspect) const;
	double GetFrameRate() const;
	int GetDVDCapacity() const;
	double GetLength() const;
	int GetMaxVideoBitRate() const;
	int GetVideoBitRate() const;
	String GetShortDescr() const;

public:
	bool Load(csv_file_t csv, const String & directory);
	void Save(FILE * f, const String & directory) const;

protected:
	int m_DVDFormat;
	int m_DVDSize;
	int m_CustomSize;
	int m_CustomBitRate;
	int m_OutputType;
	String m_OutputDirectory;
	String m_OpenFileDirectory;
	GroupVector m_Groups;
	double m_Length;
	int m_MaxVideoBitRate;
	int m_VideoBitRate;
	int m_SizeFromBitRate;
	bool m_HasChanged;
	String m_Error;
};

typedef ManagedPtr<Project> ProjectPtr;

#endif // PROJECTOBJ_H_
