#ifndef SUBTITLE_H_
#define SUBTITLE_H_

#include "managableptr.h"
#include "managedptr.h"
#include "string.h"
#include "csvread/csvread.h"

#include <vector>

#include <windows.h>

class Video;

class Subtitle : public ManagablePtr
{
public:
	Subtitle(Video * video, Subtitle * subtitle = NULL);
	virtual ~Subtitle();
	Video * GetVideo() const;
	bool IsFileSelected() const;
	void SetFileSelected(bool selected);
	bool IsTrackSelected() const;
	void SetTrackSelected(bool selected);

public:
	const String & GetFileName() const;
	bool SetFileName(const String & fileName, bool setChanged = true);
	int GetCharset() const;
	bool SetCharset(int charset, bool setChanged = true);
	const String & GetFontFace() const;
	bool SetFontFace(const String & fontFace, bool setChanged = true);
	int GetFontSize() const;
	bool SetFontSize(int fontSize, bool setChanged = true);
	bool GetBold() const;
	bool SetBold(bool bold, bool setChanged = true);
	bool GetItalic() const;
	bool SetItalic(bool italic, bool setChanged = true);
	bool GetBox() const;
	bool SetBox(bool box, bool setChanged = true);
	COLORREF GetColor() const;
	bool SetColor(COLORREF color, bool setChanged = true);
	bool GetAdvance() const;
	bool SetAdvance(bool advance, bool setChanged = true);
	int GetOffset() const;
	bool SetOffset(int offset, bool setChanged = true);
	int GetBitRateInKbps() const;
	void SetChanged();
	int GetIndex() const;
	void SetIndex(int index);
	String GetIdent(bool forTrack);
	String Recalculate();
	bool Analyze();
	bool IsChecked() const;
	bool IsValid() const;
	void GetDescription(StringVectorVector & strings, bool forTitle) const;
	bool Load(csv_file_t csv, int & lineno, int nest);
	void Save(FILE * f, int nest) const;

protected:
	Video *  m_Video;
	String   m_FileName;
	int      m_Charset;
	String   m_FontFace;
	int      m_FontSize;
	bool     m_Bold;
	bool     m_Italic;
	bool     m_Box;
	bool     m_Advance;
	int      m_Offset;
	COLORREF m_Color;
	bool     m_IsChecked;
	bool     m_IsValid;
	int      m_Index;
	bool     m_FileSelected;
	bool     m_TrackSelected;
};

typedef ManagedPtr<Subtitle> SubtitlePtr;
typedef std::vector<SubtitlePtr> SubtitleVector;

#endif // SUBTITLE_H_
