#include "subtitleentry.h"
#include "tools.h"

SubtitleEntry::SubtitleEntry()
{
	m_StartTime = 0;
	m_EndTime   = 0;
}

SubtitleEntry::~SubtitleEntry()
{
}

bool SubtitleEntry::operator < (const SubtitleEntry & entry) const
{
	return ((m_StartTime < entry.m_StartTime) ||
		    ((m_StartTime == entry.m_StartTime) && (m_EndTime < entry.m_EndTime)));
}

void SubtitleEntry::AppendLine(const String & line)
{
	m_Lines.push_back(line.trim());
}

void SubtitleEntry::AppendLines(const SubtitleEntry & entry)
{
	for (StringVector::const_iterator li = entry.m_Lines.begin(); li != entry.m_Lines.end(); li++) {
		m_Lines.push_back(*li);
	}
}

void SubtitleEntry::BreakLongestLine()
{
	if (m_Lines.size()) {
		StringVector::iterator longest = m_Lines.begin();
		for (StringVector::iterator li = longest + 1; li != m_Lines.end(); li++) {
			if ((*longest).length() < (*li).length()) {
				longest = li;
			}
		}
		String broken = (*longest);
		int middle = (int) broken.length() / 2;
		int breakat = middle;
		int breakplus = 0;
		for (int offset = 1; breakat == middle && offset < middle; offset++) {
			if (iswspace(broken[middle + offset])) {
				breakat = middle + offset;
				breakplus = 1;
			} else if (iswspace(broken[middle - offset])) {
				breakat = middle - offset;
				breakplus = 1;
			}
		}
		(*longest) = (*longest).substr(breakat + breakplus);
		m_Lines.insert(longest, broken.substr(0, breakat));
	}
}

int SubtitleEntry::GetLineCount() const
{
	return (int) m_Lines.size();
}

String SubtitleEntry::GetLine(int index) const
{
	if (0 <= index && index < (int) m_Lines.size()) {
		return m_Lines[index];
	} else {
		return String();
	}
}

void SubtitleEntry::SetStartTime(double startTime)
{
	m_StartTime = startTime;
}

void SubtitleEntry::SetEndTime(double endTime)
{
	m_EndTime = endTime;
}

double SubtitleEntry::GetStartTime() const
{
	return m_StartTime;
}

double SubtitleEntry::GetEndTime() const
{
	return m_EndTime;
}

String SubtitleEntry::GetFormattedStartTime() const
{
	return String().fromLength(m_StartTime);
}

String SubtitleEntry::GetFormattedEndTime() const
{
	return String().fromLength(m_EndTime);
}
