#include "titledialog.h"

#include "analyzedialog.h"
#include "langcode.h"
#include "main.h"
#include "resource.h"
#include "drawitem.h"
#include "tools.h"
#include "lookatvideodialog.h"
#include "config.h"

#include "winutils/winutils.h"

#define MAX_OPEN_FILES 128

typedef struct {
	String opener;
	String preProcessor;
} OpenFileExtData;

UINT APIENTRY OpenFileHookProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static OpenFileExtData * extData;
	switch (message) 
	{
	case WM_INITDIALOG:
		extData = (OpenFileExtData *) (((LPOPENFILENAME) lParam)->lCustData);
		SendDlgItemMessage(hWnd, IDC_OPENER, CB_RESETCONTENT, 0, 0);
		for (Config::FunctionMap::const_iterator fi = g_Config->GetOpeners().begin(); 
			fi != g_Config->GetOpeners().end(); fi++) {
			SendDlgItemMessage(hWnd, IDC_OPENER, CB_ADDSTRING, 0, (LPARAM) ((*fi).first.c_str()));
		}
		SendDlgItemMessage(hWnd, IDC_OPENER, CB_SETCURSEL, 
			SendDlgItemMessage(hWnd, IDC_OPENER, CB_FINDSTRINGEXACT, 0, (LPARAM) extData->opener.c_str()), 0);
		SendDlgItemMessage(hWnd, IDC_PREPROCESSOR, CB_RESETCONTENT, 0, 0);
		SendDlgItemMessage(hWnd, IDC_PREPROCESSOR, CB_ADDSTRING, 0, (LPARAM) TEXT(""));
		for (Config::FunctionMap::const_iterator fi = g_Config->GetPreProcessors().begin(); 
			fi != g_Config->GetPreProcessors().end(); fi++) {
			SendDlgItemMessage(hWnd, IDC_PREPROCESSOR, CB_ADDSTRING, 0, (LPARAM) ((*fi).first.c_str()));
		}
		SendDlgItemMessage(hWnd, IDC_PREPROCESSOR, CB_SETCURSEL, 
			SendDlgItemMessage(hWnd, IDC_PREPROCESSOR, CB_FINDSTRINGEXACT, 0, (LPARAM) extData->preProcessor.c_str()), 0);
		return 1;

	case WM_DESTROY:
		{
			int index = SendDlgItemMessage(hWnd, IDC_OPENER, CB_GETCURSEL, 0, 0);
			if (index >= 0) {
				int length = (int) SendDlgItemMessage(hWnd, IDC_OPENER, CB_GETLBTEXTLEN, index, 0);
				LPTSTR s = new TCHAR[length + 1];
				SendDlgItemMessage(hWnd, IDC_OPENER, CB_GETLBTEXT, index, (LPARAM) s);
				extData->opener = s;
				delete [] s;
			} else {
				extData->opener = S("");
			}

		} {
			int index = SendDlgItemMessage(hWnd, IDC_PREPROCESSOR, CB_GETCURSEL, 0, 0);
			if (index >= 0) {
				int length = (int) SendDlgItemMessage(hWnd, IDC_PREPROCESSOR, CB_GETLBTEXTLEN, index, 0);
				LPTSTR s = new TCHAR[length + 1];
				SendDlgItemMessage(hWnd, IDC_PREPROCESSOR, CB_GETLBTEXT, index, (LPARAM) s);
				extData->preProcessor = s;
				delete [] s;
			} else {
				extData->preProcessor = S("");
			}
		}
		return 1;
	}
	return 0;
}

int CALLBACK InitSubtitleFonts(LPENUMLOGFONT Font, LPNEWTEXTMETRIC Metric, int FontType, LPARAM lParam)
{
	HWND hWnd;
	hWnd = (HWND) lParam;
	SendDlgItemMessage(hWnd, IDC_FONTFACE, CB_ADDSTRING, 0, (LPARAM)(Font->elfLogFont.lfFaceName));
	return 1;
}

void TitleDialog::InitDialog()
{
	for (int i = 0; i < LANGUAGE_COUNT; i++) {
		SendDlgItemMessage(hWnd(), IDC_AUDIOLANGUAGE, CB_ADDSTRING, 0, (LPARAM) LanguageName[i]);
		SendDlgItemMessage(hWnd(), IDC_SUBTITLELANGUAGE, CB_ADDSTRING, 0, (LPARAM) LanguageName[i]);
	}
	SendDlgItemMessage(hWnd(), IDC_AUDIOBITRATE, CB_ADDSTRING, 0, (LPARAM) TEXT("Default"));
	SendDlgItemMessage(hWnd(), IDC_AUDIOBITRATE, CB_ADDSTRING, 0, (LPARAM) TEXT("96 kb/s"));
	SendDlgItemMessage(hWnd(), IDC_AUDIOBITRATE, CB_ADDSTRING, 0, (LPARAM) TEXT("192 kb/s"));
	SendDlgItemMessage(hWnd(), IDC_AUDIOBITRATE, CB_ADDSTRING, 0, (LPARAM) TEXT("256 kb/s"));
	SendDlgItemMessage(hWnd(), IDC_AUDIOBITRATE, CB_ADDSTRING, 0, (LPARAM) TEXT("384 kb/s"));
	SendDlgItemMessage(hWnd(), IDC_AUDIOBITRATE, CB_ADDSTRING, 0, (LPARAM) TEXT("448 kb/s"));
	for (int i = 0; i < CHARSET_COUNT; i++) {
		SendDlgItemMessage(hWnd(), IDC_CHARSET, CB_ADDSTRING, 0, (LPARAM) CharsetName[i]);
	}
	HDC dc = GetDC(hWnd());
	EnumFontFamilies(dc, NULL, (FONTENUMPROC) InitSubtitleFonts, (LPARAM) hWnd());
	ReleaseDC(hWnd(), dc);
	Refresh();
}

void TitleDialog::Refresh()
{
	SendDlgItemMessage(hWnd(), IDC_OPENVIDEOUSING, CB_RESETCONTENT, 0, 0);
	SendDlgItemMessage(hWnd(), IDC_OPENAUDIOUSING, CB_RESETCONTENT, 0, 0);
	for (Config::FunctionMap::const_iterator fi = g_Config->GetOpeners().begin(); 
		fi != g_Config->GetOpeners().end(); fi++) {
		SendDlgItemMessage(hWnd(), IDC_OPENVIDEOUSING, CB_ADDSTRING, 0, (LPARAM) ((*fi).first.c_str()));
		SendDlgItemMessage(hWnd(), IDC_OPENAUDIOUSING, CB_ADDSTRING, 0, (LPARAM) ((*fi).first.c_str()));
	}

	SendDlgItemMessage(hWnd(), IDC_PREPROCESSVIDEO, CB_RESETCONTENT, 0, 0);
	SendDlgItemMessage(hWnd(), IDC_PREPROCESSVIDEO, CB_ADDSTRING, 0, (LPARAM) TEXT(""));
	SendDlgItemMessage(hWnd(), IDC_PREPROCESSAUDIO, CB_RESETCONTENT, 0, 0);
	SendDlgItemMessage(hWnd(), IDC_PREPROCESSAUDIO, CB_ADDSTRING, 0, (LPARAM) TEXT(""));
	for (Config::FunctionMap::const_iterator fi = g_Config->GetPreProcessors().begin(); 
		fi != g_Config->GetPreProcessors().end(); fi++) {
		SendDlgItemMessage(hWnd(), IDC_PREPROCESSVIDEO, CB_ADDSTRING, 0, (LPARAM) ((*fi).first.c_str()));
		SendDlgItemMessage(hWnd(), IDC_PREPROCESSAUDIO, CB_ADDSTRING, 0, (LPARAM) ((*fi).first.c_str()));
	}
	
	Update();
}

TitleListItem::TitleListItem(Group * group, int object, int index, int indent)
{
	m_Object     = object;
	m_Index      = index;
	m_Group      = group;
	m_IsTrack    = false;
	m_Indent     = indent;
	m_NextIndent = indent;
	m_Parent     = NULL;
}

TitleListItem::TitleListItem(Title * title, int indent)
{
	m_Object     = TITLE_OBJECT;
	m_Index      = title->GetIndex();
	m_Group      = title->GetGroup();
	m_Title      = title;
	m_IsTrack    = true;
	m_Indent     = indent;
	m_NextIndent = indent;
	m_Parent     = NULL;
}

TitleListItem::TitleListItem(Video * video, bool isTrack, int indent)
{
	m_Object     = VIDEO_OBJECT;
	m_Index      = video->GetIndex();
	m_Group      = video->GetGroup();
	m_Title      = video->GetTitle();
	m_Video      = video;
	m_IsTrack    = isTrack;
	m_Indent     = indent;
	m_NextIndent = indent;
	m_Parent     = NULL;
}

TitleListItem::TitleListItem(Audio * audio, bool isTrack, int indent)
{
	m_Object     = AUDIO_OBJECT;
	m_Index      = audio->GetIndex();
	m_Group      = audio->GetVideo()->GetGroup();
	m_Title      = audio->GetVideo()->GetTitle();
	m_Audio      = audio;
	m_IsTrack    = isTrack;
	m_Indent     = indent;
	m_NextIndent = indent;
	m_Parent     = NULL;
}

TitleListItem::TitleListItem(Subtitle * subtitle, bool isTrack, int indent)
{
	m_Object     = SUBTITLE_OBJECT;
	m_Index      = subtitle->GetIndex();
	m_Group      = subtitle->GetVideo()->GetGroup();
	m_Title      = subtitle->GetVideo()->GetTitle();
	m_Subtitle   = subtitle;
	m_IsTrack    = isTrack;
	m_Indent     = indent;
	m_NextIndent = indent;
	m_Parent     = NULL;
}

TitleListItem::~TitleListItem()
{
}

int TitleListItem::GetDescription(StringVectorVector & strings) const
{
	StringVector line;
	switch (m_Object) 
	{
	case TITLE_OBJECT:
		m_Title->GetDescription(strings);
		break;
	case VIDEO_OBJECT:
		m_Video->GetDescription(strings, m_IsTrack);
		break;
	case AUDIO_OBJECT:
		m_Audio->GetDescription(strings, m_IsTrack);
		break;
	case SUBTITLE_OBJECT:
		m_Subtitle->GetDescription(strings, m_IsTrack);
		break;
	case VIDEO_TRACK:
		line.push_back(S("Video track: "));
		line.push_back(String().fromLength(m_Group->GetLength()));
		strings.push_back(line);
		break;
	case AUDIO_TRACK:
		line.push_back(S("Audio track ") + String(m_Index + 1) + S(": "));
		line.push_back(String(LanguageName[m_Group->GetAudioLanguage(m_Index)]));
		strings.push_back(line);
		break;
	case SUBTITLE_TRACK:
		line.push_back(S("Subtitle track ") + String(m_Index + 1) + S(": "));
		line.push_back(String(LanguageName[m_Group->GetSubtitleLanguage(m_Index)]));
		strings.push_back(line);
		break;
	}
	return m_Indent;
}

bool TitleListItem::IsSelected(bool really) const
{
	bool selected = false;
	switch (m_Object) 
	{
	case TITLE_OBJECT:
		selected = m_Title->GetVideo(0)->IsTitleSelected();
		break;
	case VIDEO_OBJECT:
		if (m_IsTrack) {
			selected = m_Video->IsTrackSelected();
		} else {
			selected = m_Video->IsFileSelected();
		}
		break;
	case AUDIO_OBJECT:
		if (m_IsTrack) {
			selected = m_Audio->IsTrackSelected();
		} else {
			selected = m_Audio->IsFileSelected();
		}
		break;
	case SUBTITLE_OBJECT:
		if (m_IsTrack) {
			selected = m_Subtitle->IsTrackSelected();
		} else {
			selected = m_Subtitle->IsFileSelected();
		}
		break;
	case VIDEO_TRACK:
		selected = m_Group->IsVideoTrackSelected();
		break;
	case AUDIO_TRACK:
		selected = m_Group->IsAudioTrackSelected(m_Index);
		break;
	case SUBTITLE_TRACK:
		selected = m_Group->IsSubtitleTrackSelected(m_Index);
		break;
	}
	if (selected || really) {
		return selected;
	} else if (m_Parent) {
		return m_Parent->IsSelected();
	} else {
		return false;
	}
}

void TitleListItem::SetSelected(bool selected)
{
	switch (m_Object) 
	{
	case TITLE_OBJECT:
		for (int index = 0; index < m_Title->GetVideoCount(); index++) {
			m_Title->GetVideo(index)->SetTitleSelected(selected);
		}
		break;
	case VIDEO_OBJECT:
		if (m_IsTrack) {
			m_Video->SetTrackSelected(selected);
		} else {
			m_Video->SetFileSelected(selected);
		}
		break;
	case AUDIO_OBJECT:
		if (m_IsTrack) {
			m_Audio->SetTrackSelected(selected);
		} else {
			m_Audio->SetFileSelected(selected);
		}
		break;
	case SUBTITLE_OBJECT:
		if (m_IsTrack) {
			m_Subtitle->SetTrackSelected(selected);
		} else {
			m_Subtitle->SetFileSelected(selected);
		}
		break;
	case VIDEO_TRACK:
		m_Group->SetVideoTrackSelected(selected);
		break;
	case AUDIO_TRACK:
		m_Group->SetAudioTrackSelected(m_Index, selected);
		break;
	case SUBTITLE_TRACK:
		m_Group->SetSubtitleTrackSelected(m_Index, selected);
		break;
	}
}

int TitleListItem::GetObject() const
{
	return m_Object;
}
Group * TitleListItem::GetGroup()
{
	return m_Group;
}

Title * TitleListItem::GetTitle()
{
	return m_Title;
}

Video * TitleListItem::GetVideo()
{
	return m_Video;
}

Audio * TitleListItem::GetAudio()
{
	return m_Audio;
}

void TitleListItem::SetAudio(Audio * audio)
{
	m_Audio = audio;
}

Subtitle * TitleListItem::GetSubtitle()
{
	return m_Subtitle;
}

void TitleListItem::SetSubtitle(Subtitle * subtitle)
{
	m_Subtitle = subtitle;
}

bool TitleListItem::IsTrack() const
{
	return m_IsTrack;
}

int TitleListItem::GetIndent() const
{
	return m_Indent;
}

int TitleListItem::GetNextIndent() const
{
	return m_NextIndent;
}

void TitleListItem::SetNextIndent(int nextIndent)
{
	m_NextIndent = nextIndent;
}

void TitleListItem::SetParent(TitleListItem * parent)
{
	m_Parent = parent;
}

TitleDialog::TitleDialog(MainDialog * mainDialog, GroupPtr group, int groupIndex)
{
	m_MainDialog = mainDialog;
	m_Group      = group;
	m_GroupIndex = groupIndex;
}

TitleDialog::~TitleDialog()
{
}

int TitleDialog::DoModal()
{
	return Dialog::DoModal(hInstance, MAKEINTRESOURCE(IDD_TITLE), m_MainDialog->hWnd());
}

void TitleDialog::ReconstructView()
{
	m_TitleListItems.clear();
	switch (m_Group->GetView()) 
	{
		case VIEW_TITLETRACKPART: 
		{
			for (int tindex = 0; tindex < m_Group->GetTitleCount(); tindex++) {
				Title * title = m_Group->GetTitle(tindex);
				m_TitleListItems.push_back(new TitleListItem(title, 0));
				m_TitleListItems.push_back(new TitleListItem(title->GetVideo(0), true, 1));
				for (int vindex = 0; vindex < title->GetVideoCount(); vindex++) {
					m_TitleListItems.push_back(new TitleListItem(title->GetVideo(vindex), false, 2));
				}
				for (int aindex = 0; aindex < m_Group->GetAudioCount(); aindex++) {
					m_TitleListItems.push_back(new TitleListItem(title->GetVideo(0)->GetAudio(aindex), true, 1));
					for (int vindex = 0; vindex < title->GetVideoCount(); vindex++) {
						m_TitleListItems.push_back(new TitleListItem(title->GetVideo(vindex)->GetAudio(aindex), false, 2));
					}
				}
				for (int sindex = 0; sindex < m_Group->GetSubtitleCount(); sindex++) {
					m_TitleListItems.push_back(new TitleListItem(title->GetVideo(0)->GetSubtitle(sindex), true, 1));
					for (int vindex = 0; vindex < title->GetVideoCount(); vindex++) {
						m_TitleListItems.push_back(new TitleListItem(title->GetVideo(vindex)->GetSubtitle(sindex), false, 2));
					}
				}
			}
			break;
		}
		case VIEW_TRACKTITLEPART:
		{
			m_TitleListItems.push_back(new TitleListItem(m_Group, VIDEO_TRACK, 0, 0));
			for (int tindex = 0; tindex < m_Group->GetTitleCount(); tindex++) {
				Title * title = m_Group->GetTitle(tindex);
				m_TitleListItems.push_back(new TitleListItem(title->GetVideo(0), true, 1));
				for (int vindex = 0; vindex < title->GetVideoCount(); vindex++) {
					m_TitleListItems.push_back(new TitleListItem(title->GetVideo(vindex), false, 2));
				}
			}
			for (int aindex = 0; aindex < m_Group->GetAudioCount(); aindex++) {
				m_TitleListItems.push_back(new TitleListItem(m_Group, AUDIO_TRACK, aindex, 0));
				for (int tindex = 0; tindex < m_Group->GetTitleCount(); tindex++) {
					Title * title = m_Group->GetTitle(tindex);
					m_TitleListItems.push_back(new TitleListItem(title->GetVideo(0)->GetAudio(aindex), true, 1));
					for (int vindex = 0; vindex < title->GetVideoCount(); vindex++) {
						m_TitleListItems.push_back(new TitleListItem(title->GetVideo(vindex)->GetAudio(aindex), false, 2));
					}
				}
			}
			for (int sindex = 0; sindex < m_Group->GetSubtitleCount(); sindex++) {
				m_TitleListItems.push_back(new TitleListItem(m_Group, SUBTITLE_TRACK, sindex, 0));
				for (int tindex = 0; tindex < m_Group->GetTitleCount(); tindex++) {
					Title * title = m_Group->GetTitle(tindex);
					m_TitleListItems.push_back(new TitleListItem(title->GetVideo(0)->GetSubtitle(sindex), true, 1));
					for (int vindex = 0; vindex < title->GetVideoCount(); vindex++) {
						m_TitleListItems.push_back(new TitleListItem(title->GetVideo(vindex)->GetSubtitle(sindex), false, 2));
					}
				}
			}
			break;
		}
		case VIEW_OUTLINE:
		{
			for (int tindex = 0; tindex < m_Group->GetTitleCount(); tindex++) {
				Title * title = m_Group->GetTitle(tindex);
				m_TitleListItems.push_back(new TitleListItem(title, 0));
				for (int vindex = 0; vindex < title->GetVideoCount(); vindex++) {
					m_TitleListItems.push_back(new TitleListItem(title->GetVideo(vindex), false, 1));
				}
			}
		}
		break;
	}

	TitleListItem * parent[3];
	parent[0] = parent[1] = parent[2] = NULL;
	for (TitleListItemVector::iterator tli = m_TitleListItems.begin(); tli != m_TitleListItems.end(); tli++) {
		int indent = (*tli)->GetIndent();
		if (indent) {
			(*tli)->SetParent(parent[indent - 1]);
		}
		parent[indent] = (*tli);
		if ((tli + 1) != m_TitleListItems.end()) {
			(*tli)->SetNextIndent((*(tli + 1))->GetIndent());
		}
	}
}

void TitleDialog::ClearSelection()
{
	for (TitleListItemVector::iterator tli = m_TitleListItems.begin(); tli != m_TitleListItems.end(); tli++) {
		(*tli)->SetSelected(false);
	}
}

void TitleDialog::UpdateVideoList(bool reset)
{
	SendDlgItemMessage(hWnd(), IDC_VIEW_TITLETRACKPART, BM_SETCHECK,
		(m_Group->GetView() == VIEW_TITLETRACKPART ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_VIEW_TRACKTITLEPART, BM_SETCHECK,
		(m_Group->GetView() == VIEW_TRACKTITLEPART ? BST_CHECKED : BST_UNCHECKED), 0);
	SendDlgItemMessage(hWnd(), IDC_VIEW_OUTLINE, BM_SETCHECK,
		(m_Group->GetView() == VIEW_OUTLINE ? BST_CHECKED : BST_UNCHECKED), 0);

	EnableWindow(GetDlgItem(hWnd(), IDC_ADDTITLE), m_Group->CanAddVideo());
	if (m_Group->GetView() == VIEW_OUTLINE) {
		EnableWindow(GetDlgItem(hWnd(), IDC_MOVETITLEUP),   m_Group->CanMoveSelectedVideosUp());
		EnableWindow(GetDlgItem(hWnd(), IDC_MOVETITLEDOWN), m_Group->CanMoveSelectedVideosDown());
		EnableWindow(GetDlgItem(hWnd(), IDC_DELETETITLE),   m_Group->CanDeleteSelectedVideos());
		EnableWindow(GetDlgItem(hWnd(), IDC_MERGETITLES),   m_Group->CanMergeSelectedVideos());
		EnableWindow(GetDlgItem(hWnd(), IDC_SPLITTITLES),   m_Group->CanSplitSelectedVideos());
	} else {
		EnableWindow(GetDlgItem(hWnd(), IDC_MOVETITLEUP),   FALSE);
		EnableWindow(GetDlgItem(hWnd(), IDC_MOVETITLEDOWN), FALSE);
		EnableWindow(GetDlgItem(hWnd(), IDC_DELETETITLE),   FALSE);
		EnableWindow(GetDlgItem(hWnd(), IDC_MERGETITLES),   FALSE);
		EnableWindow(GetDlgItem(hWnd(), IDC_SPLITTITLES),   FALSE);
	}

	ReconstructView();
	if (reset) ClearSelection();
	bool redraw = false;
	SendDlgItemMessage(hWnd(), IDC_TITLELIST, WM_SETREDRAW, FALSE, 0);
	if (reset) {
		SendDlgItemMessage(hWnd(), IDC_TITLELIST, LB_RESETCONTENT, 0, 0);
		redraw = true;
	}
	int listCount   = (int) SendDlgItemMessage(hWnd(), IDC_TITLELIST, LB_GETCOUNT, 0, 0);
	int topIndex    = (int) SendDlgItemMessage(hWnd(), IDC_TITLELIST, LB_GETTOPINDEX, 0, 0);
	int caretIndex  = (int) SendDlgItemMessage(hWnd(), IDC_TITLELIST, LB_GETCARETINDEX, 0, 0);
	bool firstAdded = true;
	bool firstSelected = true;
	for (int index = 0; index < (int) m_TitleListItems.size(); index++) {
		if (index < listCount) {
			SendDlgItemMessage(hWnd(), IDC_TITLELIST, LB_SETITEMDATA, (WPARAM) index, (LPARAM) index);
			MEASUREITEMSTRUCT mis;
			mis.CtlID = IDC_TITLELIST;
			mis.itemData = (UINT) index;
			MeasureItem(& mis);
			UINT origHeight = (UINT) SendDlgItemMessage(hWnd(), IDC_TITLELIST, LB_GETITEMHEIGHT, index, 0);
			if (mis.itemHeight != origHeight) {
				if (mis.itemHeight < origHeight) redraw = true;
				SendDlgItemMessage(hWnd(), IDC_TITLELIST, LB_SETITEMHEIGHT, index, MAKELPARAM(mis.itemHeight, 0));
			}
		} else {
			SendDlgItemMessage(hWnd(), IDC_TITLELIST, LB_ADDSTRING, 0, (LPARAM) index);
			if (firstAdded) {
				topIndex   = index;
				caretIndex = index;
				firstAdded = false;
			}
		}
		bool selected = m_TitleListItems[index]->IsSelected(true);
		if ((SendDlgItemMessage(hWnd(), IDC_TITLELIST, LB_GETSEL, index, 0) != 0) != selected) {
			SendDlgItemMessage(hWnd(), IDC_TITLELIST, LB_SETSEL, selected, index);
		}
		if (selected) {
			if (reset && firstSelected) {
				topIndex = index;
				caretIndex = index;
			}
			firstSelected = false;
		}
	}
	for (int index = listCount - 1; index >= (int) m_TitleListItems.size(); index--) {
		SendDlgItemMessage(hWnd(), IDC_TITLELIST, LB_DELETESTRING, index, 0);
		if (topIndex >= index) topIndex = index - 1;
		if (caretIndex >= index) caretIndex = index - 1;
		redraw = true;
	}
	SendDlgItemMessage(hWnd(), IDC_TITLELIST, LB_SETTOPINDEX, topIndex, 0);
	SendDlgItemMessage(hWnd(), IDC_TITLELIST, LB_SETCARETINDEX, caretIndex, 0);
	SendDlgItemMessage(hWnd(), IDC_TITLELIST, WM_SETREDRAW, TRUE, 0);
	InvalidateRect(GetDlgItem(hWnd(), IDC_TITLELIST), NULL, redraw);
}

void TitleDialog::UpdateVideoTitle()
{
	int aspectRatio        = ASPECTRATIO_MAX;
	int aspectNom          = -1;
	int aspectDen          = -1;
	int compensateOverscan = BST_UNCHECKED;
	int overscanPercent    = -1;
	int createChapters     = BST_UNCHECKED;
	int chapterCount       = -1;
	bool disableAspectNomDen = false;
	bool disableOverscanPercent = false;
	bool disableChapterCount = false;
	int count = 0;
	if (m_Group->GetView() != VIEW_OUTLINE) {
		for (int index = 0; index < (int) m_TitleListItems.size(); index++) {
			TitleListItem * item = m_TitleListItems[index];
			if ((item->GetObject() == VIDEO_OBJECT) && item->IsTrack() && item->IsSelected()) {
				Video * video = item->GetVideo();
				if (! count++) {
					aspectRatio        = video->GetAspectRatio();
					aspectNom          = video->GetAspectNom();
					aspectDen          = video->GetAspectDen();
					compensateOverscan = (video->GetCompensateOverscan() ? BST_CHECKED : BST_UNCHECKED);
					overscanPercent    = video->GetOverscanPercent();
					createChapters     = (video->GetCreateChapters() ? BST_CHECKED : BST_UNCHECKED);
					chapterCount       = video->GetChapterCount();
				} else {
					if (aspectRatio != video->GetAspectRatio()) 
						aspectRatio = ASPECTRATIO_MAX;
					if (aspectNom != video->GetAspectNom()) 
						aspectNom = -1;
					if (aspectDen != video->GetAspectDen()) 
						aspectDen = -1;
					if (compensateOverscan != (video->GetCompensateOverscan() ? BST_CHECKED : BST_UNCHECKED))
						compensateOverscan = BST_INDETERMINATE;
					if (overscanPercent != video->GetOverscanPercent())
						overscanPercent = -1;
					if (createChapters != (video->GetCreateChapters() ? BST_CHECKED : BST_UNCHECKED))
						createChapters = BST_INDETERMINATE;
					if (chapterCount != video->GetChapterCount()) 
						chapterCount = -1;
				}
				if (video->GetAspectRatio() != ASPECTRATIO_CUSTOM) {
					disableAspectNomDen = true;
				}
				if (! video->GetCompensateOverscan()) {
					disableOverscanPercent = true;
				}
				if (! video->GetCreateChapters()) {
					disableChapterCount = true;
				}
			}
		}
	}
	EnableWindow(GetDlgItem(hWnd(), IDC_VIDEOGROUP), count);
	EnableWindow(GetDlgItem(hWnd(), IDC_ASPECTGROUP), count);
	EnableWindow(GetDlgItem(hWnd(), IDC_ASPECT_4_3), count);
	SendDlgItemMessage(hWnd(), IDC_ASPECT_4_3, BM_SETCHECK,
		(aspectRatio == ASPECTRATIO_4_3 ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_ASPECT_16_9), count);
	SendDlgItemMessage(hWnd(), IDC_ASPECT_16_9, BM_SETCHECK,
		(aspectRatio == ASPECTRATIO_16_9 ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_ASPECT_2_1), count);
	SendDlgItemMessage(hWnd(), IDC_ASPECT_2_1, BM_SETCHECK,
		(aspectRatio == ASPECTRATIO_2_1 ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_ASPECT_239_1), count);
	SendDlgItemMessage(hWnd(), IDC_ASPECT_239_1, BM_SETCHECK,
		(aspectRatio == ASPECTRATIO_239_1 ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_ASPECT_CUSTOM), count);
	SendDlgItemMessage(hWnd(), IDC_ASPECT_CUSTOM, BM_SETCHECK,
		(aspectRatio == ASPECTRATIO_CUSTOM ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_ASPECTNOM), count && ! disableAspectNomDen);
	SetDlgItemText(hWnd(), IDC_ASPECTNOM, 
		((disableAspectNomDen || aspectNom <= 0) ? TEXT("") : String(aspectNom).c_str()));
	EnableWindow(GetDlgItem(hWnd(), IDC_ASPECTCOLON), count && ! disableAspectNomDen);
	EnableWindow(GetDlgItem(hWnd(), IDC_ASPECTDEN), count && ! disableAspectNomDen);
	SetDlgItemText(hWnd(), IDC_ASPECTDEN, 
		((disableAspectNomDen || aspectDen <= 0) ? TEXT("") : String(aspectDen).c_str()));
	EnableWindow(GetDlgItem(hWnd(), IDC_COMPENSATEOVERSCAN), count);
	SendDlgItemMessage(hWnd(), IDC_COMPENSATEOVERSCAN, BM_SETCHECK, compensateOverscan, 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_OVERSCANPERCENT), count && ! disableOverscanPercent);
	SetDlgItemText(hWnd(), IDC_OVERSCANPERCENT,
		((disableOverscanPercent || (overscanPercent <= 0)) ? TEXT("") : String(overscanPercent).c_str()));
	EnableWindow(GetDlgItem(hWnd(), IDC_OVERSCANSUFFIX), count);
	EnableWindow(GetDlgItem(hWnd(), IDC_CREATECHAPTERS), count);
	SendDlgItemMessage(hWnd(), IDC_CREATECHAPTERS, BM_SETCHECK, createChapters, 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_CHAPTERCOUNT), count && ! disableChapterCount);
	SetDlgItemText(hWnd(), IDC_CHAPTERCOUNT,
		((disableChapterCount || (chapterCount <= 0)) ? TEXT("") : String(chapterCount).c_str()));
	EnableWindow(GetDlgItem(hWnd(), IDC_CHAPTERSUFFIX), count);
}

void TitleDialog::UpdateVideoPart()
{
	String fileName;
	int opener        = -1;
	int preProcessor  = -1;
	int upsideDown    = BST_UNCHECKED;
	int interlacing   = INTERLACING_MAX;
	int doIVTC        = BST_UNCHECKED;
	int fixedChapters = BST_UNCHECKED;
	String chaptersAt;
	bool disableOpener       = false;
	bool disableDoIVTC       = false;
	bool disableChaptersAt   = false;
	int count = 0;
	if (m_Group->GetView() != VIEW_OUTLINE) {
		for (int index = 0; index < (int) m_TitleListItems.size(); index++) {
			TitleListItem * item = m_TitleListItems[index];
			if ((item->GetObject() == VIDEO_OBJECT) && ! item->IsTrack() && item->IsSelected()) {
				Video * video = item->GetVideo();
				if (! count++) {
					fileName      = video->GetFileName();
					opener        = (int) SendDlgItemMessage(hWnd(), IDC_OPENVIDEOUSING, CB_FINDSTRINGEXACT, 0, (LPARAM) video->GetOpener().c_str());
					preProcessor  = (int) SendDlgItemMessage(hWnd(), IDC_PREPROCESSVIDEO, CB_FINDSTRINGEXACT, 0, (LPARAM) video->GetPreProcessor().c_str());
					upsideDown    = (video->GetUpsideDown() ? BST_CHECKED : BST_UNCHECKED);
					interlacing   = video->GetInterlacing();
					doIVTC        = (video->GetDoIVTC() ? BST_CHECKED : BST_UNCHECKED);
					fixedChapters = (video->GetFixedChapters() ? BST_CHECKED : BST_UNCHECKED);
					chaptersAt    = video->GetChaptersAt();
				} else {
					fileName += S("; ") + video->GetFileName();
					if (opener != (int) SendDlgItemMessage(hWnd(), IDC_OPENVIDEOUSING, CB_FINDSTRINGEXACT, 0, (LPARAM) video->GetOpener().c_str())) 
						opener = -1;
					if (preProcessor != (int) SendDlgItemMessage(hWnd(), IDC_PREPROCESSVIDEO, CB_FINDSTRINGEXACT, 0, (LPARAM) video->GetPreProcessor().c_str()))
						preProcessor = -1;
					if (upsideDown != (video->GetUpsideDown() ? BST_CHECKED : BST_UNCHECKED)) 
						upsideDown = BST_INDETERMINATE;
					if (interlacing != video->GetInterlacing())
						interlacing = INTERLACING_MAX;
					if (doIVTC != (video->GetDoIVTC() ? BST_CHECKED : BST_UNCHECKED))
						doIVTC = BST_INDETERMINATE;
					if (fixedChapters != (video->GetFixedChapters() ? BST_CHECKED : BST_UNCHECKED))
						fixedChapters = BST_INDETERMINATE;
					if (chaptersAt != video->GetChaptersAt())
						chaptersAt = S("");
				}
				if (video->IsScript()) {
					opener = -1;
					disableOpener = true;
				}
				if (video->GetInterlacing() <= INTERLACING_PROGRESSIVE) {
					disableDoIVTC = true;
				}
				if (! video->GetFixedChapters()) {
					disableChaptersAt = true;
				}
			}
		}
	}
	EnableWindow(GetDlgItem(hWnd(), IDC_VIDEOPARTGROUP), count);
	EnableWindow(GetDlgItem(hWnd(), IDC_VIDEOFILE), count);
	SetDlgItemText(hWnd(), IDC_VIDEOFILE, fileName.c_str());
	EnableWindow(GetDlgItem(hWnd(), IDC_OPENVIDEOUSINGPREFIX), count && ! disableOpener);
	EnableWindow(GetDlgItem(hWnd(), IDC_OPENVIDEOUSING), count && ! disableOpener);
	SendDlgItemMessage(hWnd(), IDC_OPENVIDEOUSING, CB_SETCURSEL, opener, 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_PREPROCESSVIDEOPREFIX), count);
	EnableWindow(GetDlgItem(hWnd(), IDC_PREPROCESSVIDEO), count);
	SendDlgItemMessage(hWnd(), IDC_PREPROCESSVIDEO, CB_SETCURSEL, preProcessor, 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_LOOKATVIDEO), count && fileName.length());
	EnableWindow(GetDlgItem(hWnd(), IDC_INTERLACINGROUP), count);
	EnableWindow(GetDlgItem(hWnd(), IDC_UPSIDEDOWN), count);
	SendDlgItemMessage(hWnd(), IDC_UPSIDEDOWN, BM_SETCHECK, upsideDown, 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_AUTODEINTERLACE), count);
	SendDlgItemMessage(hWnd(), IDC_AUTODEINTERLACE, BM_SETCHECK,
		((interlacing == INTERLACING_AUTO) ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_PROGRESSIVE), count);
	SendDlgItemMessage(hWnd(), IDC_PROGRESSIVE, BM_SETCHECK,
		((interlacing == INTERLACING_PROGRESSIVE) ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_INTERLACED_TFF), count);
	SendDlgItemMessage(hWnd(), IDC_INTERLACED_TFF, BM_SETCHECK,
		((interlacing == INTERLACING_TFF) ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_INTERLACED_BFF), count);
	SendDlgItemMessage(hWnd(), IDC_INTERLACED_BFF, BM_SETCHECK,
		((interlacing == INTERLACING_BFF) ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_DOIVTC), count && ! disableDoIVTC);
	SendDlgItemMessage(hWnd(), IDC_DOIVTC, BM_SETCHECK, doIVTC, 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_FIXEDCHAPTERS), count);
	SendDlgItemMessage(hWnd(), IDC_FIXEDCHAPTERS, BM_SETCHECK, fixedChapters, 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_CHAPTERSAT), count && ! disableChaptersAt);
	SetDlgItemText(hWnd(), IDC_CHAPTERSAT, (disableChaptersAt ? TEXT("") : chaptersAt.c_str()));
}

void TitleDialog::UpdateAudioList()
{
	int audioIndex = m_Group->GetSelectedAudio();
	EnableWindow(GetDlgItem(hWnd(), IDC_ADDAUDIO),      m_Group->CanAddAudio());
	EnableWindow(GetDlgItem(hWnd(), IDC_MOVEAUDIOUP),   m_Group->CanMoveAudioUp(audioIndex));
	EnableWindow(GetDlgItem(hWnd(), IDC_MOVEAUDIODOWN), m_Group->CanMoveAudioDown(audioIndex));
	EnableWindow(GetDlgItem(hWnd(), IDC_DELETEAUDIO),   m_Group->CanDeleteAudio(audioIndex));
	SendDlgItemMessage(hWnd(), IDC_AUDIOLIST, WM_SETREDRAW, FALSE, 0);
	int listCount  = (int) SendDlgItemMessage(hWnd(), IDC_AUDIOLIST, LB_GETCOUNT, 0, 0);
	int topIndex   = (int) SendDlgItemMessage(hWnd(), IDC_AUDIOLIST, LB_GETTOPINDEX, 0, 0);
	bool firstAdded = true;
	for (int index = 0; index < m_Group->GetAudioCount(); index++) {
		if (index < listCount) {
			SendDlgItemMessage(hWnd(), IDC_AUDIOLIST, LB_SETITEMDATA, index, 
				(LPARAM) m_Group->GetAudioLanguage(index));
		} else {
			SendDlgItemMessage(hWnd(), IDC_AUDIOLIST, LB_ADDSTRING, 0, 
				(LPARAM) m_Group->GetAudioLanguage(index));
			if (firstAdded) {
				topIndex   = index;
				firstAdded = false;
			}
		}
	}
	for (int index = listCount - 1; index >= m_Group->GetAudioCount(); index--) {
		SendDlgItemMessage(hWnd(), IDC_AUDIOLIST, LB_DELETESTRING, index, 0);
	}
	SendDlgItemMessage(hWnd(), IDC_AUDIOLIST, LB_SETTOPINDEX, topIndex, 0);
	SendDlgItemMessage(hWnd(), IDC_AUDIOLIST, LB_SETCURSEL, audioIndex, 0);
	SendDlgItemMessage(hWnd(), IDC_AUDIOLIST, WM_SETREDRAW, TRUE, 0);
	InvalidateRect(GetDlgItem(hWnd(), IDC_AUDIOLIST), NULL, FALSE);
	if (m_Group->IsAudioIndexValid(audioIndex)) {
		EnableWindow(GetDlgItem(hWnd(), IDC_AUDIOLANGPREFIX), TRUE);
		SetDlgItemText(hWnd(), IDC_AUDIOLANGPREFIX, (S("Language of audio track ") + String(audioIndex + 1) + S(":")).c_str());
		EnableWindow(GetDlgItem(hWnd(), IDC_AUDIOLANGUAGE),   TRUE);
		SendDlgItemMessage(hWnd(), IDC_AUDIOLANGUAGE, CB_SETCURSEL, m_Group->GetAudioLanguage(audioIndex), 0);
	} else {
		EnableWindow(GetDlgItem(hWnd(), IDC_AUDIOLANGPREFIX), FALSE);
		SetDlgItemText(hWnd(), IDC_AUDIOLANGPREFIX, TEXT("Language of audio track:"));
		EnableWindow(GetDlgItem(hWnd(), IDC_AUDIOLANGUAGE),   FALSE);
		SendDlgItemMessage(hWnd(), IDC_AUDIOLANGUAGE, CB_SETCURSEL, (WPARAM) -1, 0);
	}
}

void TitleDialog::UpdateAudioTitle()
{
	int bitrate = -1;
	int count = 0;
	if (m_Group->GetView() != VIEW_OUTLINE) {
		for (int index = 0; index < (int) m_TitleListItems.size(); index++) {
			TitleListItem * item = m_TitleListItems[index];
			if ((item->GetObject() == AUDIO_OBJECT) && item->IsTrack() && item->IsSelected()) {
				Audio * audio = item->GetAudio();
				if (! count++) {
					bitrate = audio->GetBitRate();
				} else {
					if (bitrate != audio->GetBitRate()) 
						bitrate = -1;
				}
			}
		}
	}
	EnableWindow(GetDlgItem(hWnd(), IDC_AUDIOGROUP), count);
	EnableWindow(GetDlgItem(hWnd(), IDC_BITRATEPREFIX), count);
	EnableWindow(GetDlgItem(hWnd(), IDC_AUDIOBITRATE), count);
	SendDlgItemMessage(hWnd(), IDC_AUDIOBITRATE, CB_SETCURSEL, bitrate, 0);
}

void TitleDialog::UpdateAudioPart()
{
	int fromVideo    = -1;
	String fileName;
	bool fromFile    = false;
	int opener       = -1;
	int preProcessor = -1;
	int count = 0;
	bool disableOpener = false;
	if (m_Group->GetView() != VIEW_OUTLINE) {
		for (int index = 0; index < (int) m_TitleListItems.size(); index++) {
			TitleListItem * item = m_TitleListItems[index];
			if ((item->GetObject() == AUDIO_OBJECT) && ! item->IsTrack() && item->IsSelected()) {
				Audio * audio = item->GetAudio();
				if (! count++) {
					fromVideo     = (audio->GetFromVideo() ? 1 : 0);
					fileName      = audio->GetFileName();
					opener        = (int) SendDlgItemMessage(hWnd(), IDC_OPENAUDIOUSING, CB_FINDSTRINGEXACT, 0, (LPARAM) audio->GetOpener().c_str());
					preProcessor  = (int) SendDlgItemMessage(hWnd(), IDC_PREPROCESSAUDIO, CB_FINDSTRINGEXACT, 0, (LPARAM) audio->GetPreProcessor().c_str());
				} else {
					if (fromVideo != (audio->GetFromVideo() ? 1 : 0))
						fromVideo = -1;
					if (! audio->GetFromVideo())
						fileName += S("; ") + audio->GetFileName();
					if (opener != (int) SendDlgItemMessage(hWnd(), IDC_OPENAUDIOUSING, CB_FINDSTRINGEXACT, 0, (LPARAM) audio->GetOpener().c_str()))
						opener = -1;
					if (preProcessor != (int) SendDlgItemMessage(hWnd(), IDC_PREPROCESSAUDIO, CB_FINDSTRINGEXACT, 0, (LPARAM) audio->GetPreProcessor().c_str()))
						preProcessor = -1;
				}
				if (! audio->GetFromVideo()) fromFile = true;
				if (audio->IsScript() || audio->GetFromVideo()) {
					opener = -1;
					disableOpener = true;
				}
			}
		}
	}
	EnableWindow(GetDlgItem(hWnd(), IDC_AUDIOPARTGROUP), count);
	EnableWindow(GetDlgItem(hWnd(), IDC_AUDIOFROMVIDEO), count);
	SendDlgItemMessage(hWnd(), IDC_AUDIOFROMVIDEO, BM_SETCHECK,
		(fromVideo == 1 ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_AUDIOFROMFILE), count);
	SendDlgItemMessage(hWnd(), IDC_AUDIOFROMFILE, BM_SETCHECK,
		(fromVideo == 0 ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_AUDIOFILE), fromFile);
	SetDlgItemText(hWnd(), IDC_AUDIOFILE, fileName.c_str());
	EnableWindow(GetDlgItem(hWnd(), IDC_BROWSEAUDIO), fromFile);
	EnableWindow(GetDlgItem(hWnd(), IDC_OPENAUDIOUSINGPREFIX), count && ! disableOpener);
	EnableWindow(GetDlgItem(hWnd(), IDC_OPENAUDIOUSING), count && ! disableOpener);
	SendDlgItemMessage(hWnd(), IDC_OPENAUDIOUSING, CB_SETCURSEL, opener, 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_PREPROCESSAUDIOPREFIX), count);
	EnableWindow(GetDlgItem(hWnd(), IDC_PREPROCESSAUDIO), count);
	SendDlgItemMessage(hWnd(), IDC_PREPROCESSAUDIO, CB_SETCURSEL, preProcessor, 0);
}

void TitleDialog::UpdateSubtitleList()
{
	int subtitleIndex = m_Group->GetSelectedSubtitle();
	EnableWindow(GetDlgItem(hWnd(), IDC_ADDSUBTITLE),        m_Group->CanAddSubtitle());
	EnableWindow(GetDlgItem(hWnd(), IDC_MOVESUBTITLEUP),     m_Group->CanMoveSubtitleUp(subtitleIndex));
	EnableWindow(GetDlgItem(hWnd(), IDC_MOVESUBTITLEDOWN),   m_Group->CanMoveSubtitleDown(subtitleIndex));
	EnableWindow(GetDlgItem(hWnd(), IDC_DELETESUBTITLE),     m_Group->CanDeleteSubtitle(subtitleIndex));
	SendDlgItemMessage(hWnd(), IDC_SUBTITLELIST, WM_SETREDRAW, FALSE, 0);
	int listCount  = (int) SendDlgItemMessage(hWnd(), IDC_SUBTITLELIST, LB_GETCOUNT, 0, 0);
	int topIndex   = (int) SendDlgItemMessage(hWnd(), IDC_SUBTITLELIST, LB_GETTOPINDEX, 0, 0);
	bool firstAdded = true;
	for (int index = 0; index < m_Group->GetSubtitleCount(); index++) {
		if (index < listCount) {
			SendDlgItemMessage(hWnd(), IDC_SUBTITLELIST, LB_SETITEMDATA, index, 
				(LPARAM) m_Group->GetSubtitleLanguage(index));
		} else {
			SendDlgItemMessage(hWnd(), IDC_SUBTITLELIST, LB_ADDSTRING, 0, 
				(LPARAM) m_Group->GetSubtitleLanguage(index));
			if (firstAdded) {
				topIndex   = index;
				firstAdded = false;
			}
		}
	}
	for (int index = listCount - 1; index >= m_Group->GetSubtitleCount(); index--) {
		SendDlgItemMessage(hWnd(), IDC_SUBTITLELIST, LB_DELETESTRING, index, 0);
	}
	SendDlgItemMessage(hWnd(), IDC_SUBTITLELIST, LB_SETTOPINDEX, topIndex, 0);
	SendDlgItemMessage(hWnd(), IDC_SUBTITLELIST, LB_SETCURSEL, subtitleIndex, 0);
	SendDlgItemMessage(hWnd(), IDC_SUBTITLELIST, WM_SETREDRAW, TRUE, 0);
	InvalidateRect(GetDlgItem(hWnd(), IDC_SUBTITLELIST), NULL, FALSE);
	if (m_Group->IsSubtitleIndexValid(subtitleIndex)) {
		EnableWindow(GetDlgItem(hWnd(), IDC_SUBTITLELANGPREFIX), TRUE);
		SetDlgItemText(hWnd(), IDC_SUBTITLELANGPREFIX, (S("Language of subtitle track ") + String(subtitleIndex + 1) + S(":")).c_str());
		EnableWindow(GetDlgItem(hWnd(), IDC_SUBTITLELANGUAGE),   TRUE);
		SendDlgItemMessage(hWnd(), IDC_SUBTITLELANGUAGE, CB_SETCURSEL, m_Group->GetSubtitleLanguage(subtitleIndex), 0);
	} else {
		EnableWindow(GetDlgItem(hWnd(), IDC_SUBTITLELANGPREFIX), FALSE);
		SetDlgItemText(hWnd(), IDC_SUBTITLELANGPREFIX, TEXT("Language of subtitle track:"));
		EnableWindow(GetDlgItem(hWnd(), IDC_SUBTITLELANGUAGE),   FALSE);
		SendDlgItemMessage(hWnd(), IDC_SUBTITLELANGUAGE, CB_SETCURSEL, (WPARAM) -1, 0);
	}
	EnableWindow(GetDlgItem(hWnd(), IDC_FIRSTSUBTITLEON), m_Group->GetSubtitleCount());
	SendDlgItemMessage(hWnd(), IDC_FIRSTSUBTITLEON, BM_SETCHECK, 
		(m_Group->GetSubtitleCount() && m_Group->GetFirstSubtitleOn() ? BST_CHECKED : BST_UNCHECKED), 0);
}

void TitleDialog::UpdateSubtitleTitle()
{
	String fontFace;
	int fontSize = -1;
	int bold     = BST_UNCHECKED;
	int italic   = BST_UNCHECKED;
	int box      = BST_UNCHECKED;
	int color    = -1;
	int count = 0;
	if (m_Group->GetView() != VIEW_OUTLINE) {
		for (int index = 0; index < (int) m_TitleListItems.size(); index++) {
			TitleListItem * item = m_TitleListItems[index];
			if ((item->GetObject() == SUBTITLE_OBJECT) && item->IsTrack() && item->IsSelected()) {
				Subtitle * subtitle = item->GetSubtitle();
				if (! count++) {
					fontFace = subtitle->GetFontFace();
					fontSize = subtitle->GetFontSize();
					bold     = (subtitle->GetBold() ? BST_CHECKED : BST_UNCHECKED);
					italic   = (subtitle->GetItalic() ? BST_CHECKED : BST_UNCHECKED);
					box      = (subtitle->GetBox() ? BST_CHECKED : BST_UNCHECKED);
					color    = subtitle->GetColor();
				} else {
					if (fontFace != subtitle->GetFontFace()) 
						fontFace = S("");
					if (fontSize != subtitle->GetFontSize())
						fontSize = -1;
					if (bold != (subtitle->GetBold() ? BST_CHECKED : BST_UNCHECKED))
						bold = BST_INDETERMINATE;
					if (italic != (subtitle->GetItalic() ? BST_CHECKED : BST_UNCHECKED))
						italic = BST_INDETERMINATE;
					if (box != (subtitle->GetBox() ? BST_CHECKED : BST_UNCHECKED))
						box = BST_INDETERMINATE;
					if (color != subtitle->GetColor())
						color = -1;
				}
			}
		}
	}
	EnableWindow(GetDlgItem(hWnd(), IDC_SUBTITLEGROUP), count);
	EnableWindow(GetDlgItem(hWnd(), IDC_FONTPREFIX), count);
	EnableWindow(GetDlgItem(hWnd(), IDC_FONTFACE), count);
	SetDlgItemText(hWnd(), IDC_FONTFACE, fontFace.c_str());
	EnableWindow(GetDlgItem(hWnd(), IDC_SIZEPREFIX), count);
	EnableWindow(GetDlgItem(hWnd(), IDC_FONTSIZE), count);
	SetDlgItemText(hWnd(), IDC_FONTSIZE, ((fontSize >= 0) ? String(fontSize).c_str() : TEXT("")));
	EnableWindow(GetDlgItem(hWnd(), IDC_SIZESUFFIX), count);
	EnableWindow(GetDlgItem(hWnd(), IDC_BOLD), count);
	SendDlgItemMessage(hWnd(), IDC_BOLD, BM_SETCHECK, bold, 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_ITALIC), count);
	SendDlgItemMessage(hWnd(), IDC_ITALIC, BM_SETCHECK, italic, 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_BOX), count);
	SendDlgItemMessage(hWnd(), IDC_BOX, BM_SETCHECK, box, 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_COLORPREFIX), count);
	EnableWindow(GetDlgItem(hWnd(), IDC_VIEWCOLOR), count);
	SetWindowLong(GetDlgItem(hWnd(), IDC_VIEWCOLOR), GWL_USERDATA, color);
	InvalidateRect(GetDlgItem(hWnd(), IDC_VIEWCOLOR), NULL, FALSE);
	EnableWindow(GetDlgItem(hWnd(), IDC_CHOOSECOLOR), count);
}

void TitleDialog::UpdateSubtitlePart()
{
	String fileName;
	int charset = -1;
	int advance = -1;
	int offset  = -1;
	int count = 0;
	if (m_Group->GetView() != VIEW_OUTLINE) {
		for (int index = 0; index < (int) m_TitleListItems.size(); index++) {
			TitleListItem * item = m_TitleListItems[index];
			if ((item->GetObject() == SUBTITLE_OBJECT) && ! item->IsTrack() && item->IsSelected()) {
				Subtitle * subtitle = item->GetSubtitle();
				if (! count++) {
					fileName = subtitle->GetFileName();
					charset  = subtitle->GetCharset();
					advance  = (subtitle->GetAdvance() ? 1 : 0);
					offset   = subtitle->GetOffset();
				} else {
					fileName += S("; ") + subtitle->GetFileName();
					if (charset != subtitle->GetCharset())
						charset = -1;
					if (advance != (subtitle->GetAdvance() ? 1 : 0))
						advance = -1;
					if (offset != subtitle->GetOffset())
						offset = -1;
				}
			}
		}
	}
	EnableWindow(GetDlgItem(hWnd(), IDC_SUBTITLEPARTGROUP), count);
	EnableWindow(GetDlgItem(hWnd(), IDC_SUBTITLEFILE), count);
	SetDlgItemText(hWnd(), IDC_SUBTITLEFILE, fileName.c_str());
	EnableWindow(GetDlgItem(hWnd(), IDC_BROWSESUBTITLE), count);
	EnableWindow(GetDlgItem(hWnd(), IDC_CHARSETPREFIX), count);
	EnableWindow(GetDlgItem(hWnd(), IDC_CHARSET), count);
	SendDlgItemMessage(hWnd(), IDC_CHARSET, CB_SETCURSEL, charset, 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_DELAYSUBTITLE), count);
	SendDlgItemMessage(hWnd(), IDC_DELAYSUBTITLE, BM_SETCHECK,
		((advance == 0) ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_ADVANCESUBTITLE), count);
	SendDlgItemMessage(hWnd(), IDC_ADVANCESUBTITLE, BM_SETCHECK,
		((advance == 1) ? BST_CHECKED : BST_UNCHECKED), 0);
	EnableWindow(GetDlgItem(hWnd(), IDC_SUBTITLEOFFSET), count);
	SetDlgItemText(hWnd(), IDC_SUBTITLEOFFSET, ((offset >= 0) ? String(offset).c_str() : TEXT("")));
	EnableWindow(GetDlgItem(hWnd(), IDC_SOFFSETSUFFIX), count);
}

void TitleDialog::Update(bool reset)
{
	String dialogTitle = S("Edit videos, audio tracks and subtitles of group ") + String(m_GroupIndex + 1);
	SetWindowText(hWnd(), dialogTitle.c_str());

	UpdateVideoList(reset);
	UpdateVideoTitle();
	UpdateVideoPart();
	UpdateAudioList();
	UpdateAudioTitle();
	UpdateAudioPart();
	UpdateSubtitleList();
	UpdateSubtitleTitle();
	UpdateSubtitlePart();

	m_MainDialog->Update();
}

void TitleDialog::MeasureItem(LPMEASUREITEMSTRUCT mis)
{
	int index  = (int) mis->itemData;
	int indent = 0;
	StringVectorVector strings;
	switch (mis->CtlID) {
	case IDC_TITLELIST:
		indent = m_TitleListItems[index]->GetDescription(strings);
		break;
	case IDC_AUDIOLIST:
	case IDC_SUBTITLELIST:
		strings.push_back(StringVector());
		strings[0].push_back(String());
		strings[0].push_back(LanguageName[index]);
		break;
	}
	MeasureListItem(hWnd(), mis, strings, 0);
}
void TitleDialog::DrawItem(LPDRAWITEMSTRUCT dis)
{
	int index = (int) dis->itemData;
	StringVectorVector strings;
	int  listIndex   = 0;
	int  listIndent  = 0;
	int  nextIndent  = 0;
	bool overridesel = false;

	switch (dis->CtlID) {
	case IDC_TITLELIST:
		if (0 <= index && index < (int) m_TitleListItems.size()) {
			listIndent = m_TitleListItems[index]->GetDescription(strings);
			nextIndent = m_TitleListItems[index]->GetNextIndent();
			overridesel = m_TitleListItems[index]->IsSelected();
		}
		break;
	case IDC_AUDIOLIST:
	case IDC_SUBTITLELIST:
		if (0 <= index && index < LANGUAGE_COUNT) {
			listIndex = (int) dis->itemID + 1;
			strings.push_back(StringVector());
			strings[0].push_back(String());
			strings[0].push_back(LanguageName[index]);
		}
		break;
	}
	DrawListItem(hWnd(), dis, strings, listIndex, listIndent, nextIndent, overridesel);
}

void TitleDialog::OnSelChange()
{
	for (int index = 0; index < (int) m_TitleListItems.size(); index++) {
		bool selected = (SendDlgItemMessage(hWnd(), IDC_TITLELIST, LB_GETSEL, index, 0) != 0);
		m_TitleListItems[index]->SetSelected(selected);
	}
	int selectedAudio = (int) SendDlgItemMessage(hWnd(), IDC_AUDIOLIST, LB_GETCURSEL, 0, 0);
	m_Group->SetSelectedAudio(selectedAudio);
	int selectedSubtitle = (int) SendDlgItemMessage(hWnd(), IDC_SUBTITLELIST, LB_GETCURSEL, 0, 0);
	m_Group->SetSelectedSubtitle(selectedSubtitle);
	m_Group->Recalculate();
	Update();
}

void TitleDialog::AddTitle()
{
	TCHAR fileNames[MAX_PATH * MAX_OPEN_FILES] = TEXT("");
	OpenFileExtData openData;
	openData.opener = g_Config->GetVideoOpener();
	openData.preProcessor = g_Config->GetVideoPreProcessor();
	OPENFILENAME ofn;
	ZeroMemory(&ofn, sizeof(OPENFILENAME));
	ofn.lStructSize     = sizeof(OPENFILENAME);
	ofn.hwndOwner       = hWnd();
	ofn.hInstance       = hInstance;
	ofn.lpstrFilter     = TEXT("Video files\0")
		TEXT("*.avs;*.asf;*.asx;*.avi;*.divx;*.dv;*.flv;*.grf;*.m1v;*.m2v;*.m4e;*.mov;*.mp4;*.mpe;*.mpeg;*.mpg;*.mkv;*.mov;*.ogm;*.qt;*.rm;*.rmvb;*.wmv\0")
		TEXT("All files\0*.*\0\0");
	ofn.lpstrFile       = fileNames;
	ofn.nMaxFile        = MAX_PATH * MAX_OPEN_FILES;
	ofn.lpstrInitialDir = m_Group->GetProject()->GetOpenFileDirectory().c_str();
	ofn.lpstrTitle      = TEXT("Add video files");
	ofn.Flags           = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_ALLOWMULTISELECT | OFN_EXPLORER | OFN_ENABLETEMPLATE | OFN_ENABLEHOOK | OFN_ENABLESIZING;
	ofn.lpTemplateName  = MAKEINTRESOURCE(IDD_OPENFILEEXT);
	ofn.lpfnHook        = OpenFileHookProc;
	ofn.lCustData       = (LPARAM) & openData;
	if (GetOpenFileName(& ofn)) {
		int fileCount;
		LPTSTR * fileList = GetOpenFileList(fileNames, & fileCount);
		m_Group->GetProject()->SetOpenFileDirectory(String(fileList[0]).replaceFileName(S("")));
		AnalyzeDialog analyzeDialog;
		for (int i = 0; i < fileCount; i++) {
			VideoPtr video = VideoPtr(new Video(m_Group));
			video->SetFileName(fileList[i], false);
			video->SetOpener(openData.opener, false);
			video->SetPreProcessor(openData.preProcessor, false);
			analyzeDialog.AddVideo(video);
		}
		FreeOpenFileList(fileList);
		ExecuteMessageLoop();
		if (analyzeDialog.DoModal(this)) {
			ClearSelection();
			for (int i = 0; i < analyzeDialog.GetCount(); i++) {
				VideoPtr video = analyzeDialog.GetVideo(i);
				m_Group->AddVideo(video, true);
				if (video->HasAudio()) {
					if (! m_Group->GetAudioCount()) {
						m_Group->AddAudio(0);
					}
					video->GetAudio(0)->SetFromVideo(true, false);
					video->GetAudio(0)->Analyze();
				}
			}
			Update();
		} else {
			String errorMessage;
			for (int i = 0; i < analyzeDialog.GetCount(); i++) {
				if (! analyzeDialog.GetVideo(i)->IsValid()) {
					errorMessage += analyzeDialog.GetVideo(i)->Recalculate();
				}
			}
			MessageBox(hWnd(), errorMessage.trim().c_str(), APPLICATION, MB_OK | MB_ICONSTOP);
		}
	}
}

bool TitleDialog::DialogProc(UINT message, WPARAM wParam, LPARAM lParam)
{
	static bool hasChanged = false;

	switch (message) {
	case WM_INITDIALOG:
		InitDialog();
		return true;

	case WM_COMMAND:
		switch (LOWORD(wParam)) {
			
		case IDC_TITLELIST:
		case IDC_AUDIOLIST:
		case IDC_SUBTITLELIST:
			if (HIWORD(wParam) == LBN_SELCHANGE) {
				OnSelChange();
				return true;
			}
			return false;

		case IDC_VIEW_TITLETRACKPART:
			if (HIWORD(wParam) == BN_CLICKED) {
				if (m_Group->GetView() != IDC_VIEW_TITLETRACKPART) {
					m_Group->SetView(VIEW_TITLETRACKPART);
					Update(true);
				}
				return true;
			}
			return false;

		case IDC_VIEW_TRACKTITLEPART:
			if (HIWORD(wParam) == BN_CLICKED) {
				if (m_Group->GetView() != VIEW_TRACKTITLEPART) {
					m_Group->SetView(VIEW_TRACKTITLEPART);
					Update(true);
				}
				return true;
			}
			return false;

		case IDC_VIEW_OUTLINE:
			if (HIWORD(wParam) == BN_CLICKED) {
				if (m_Group->GetView() != IDC_VIEW_OUTLINE) {
					m_Group->SetView(VIEW_OUTLINE);
					Update(true);
				}
				return true;
			}
			return false;

		case IDC_ADDTITLE:
			if (HIWORD(wParam) == BN_CLICKED) {
				AddTitle();
				return true;
			}
			return false;

		case IDC_MOVETITLEUP:
			if (HIWORD(wParam) == BN_CLICKED) {
				m_Group->MoveSelectedVideosUp();
				Update();
				return true;
			}
			return false;

		case IDC_MOVETITLEDOWN:
			if (HIWORD(wParam) == BN_CLICKED) {
				m_Group->MoveSelectedVideosDown();
				Update();
				return true;
			}
			return false;

		case IDC_DELETETITLE:
			if (HIWORD(wParam) == BN_CLICKED) {
				m_Group->DeleteSelectedVideos();
				Update();
				return true;
			}
			return false;

		case IDC_MERGETITLES:
			if (HIWORD(wParam) == BN_CLICKED) {
				m_Group->MergeSelectedVideos();
				Update();
				return true;
			}
			return false;

		case IDC_SPLITTITLES:
			if (HIWORD(wParam) == BN_CLICKED) {
				m_Group->SplitSelectedVideos();
				Update();
				return true;
			}
			return false;

		case IDC_ADDAUDIO:
			if (HIWORD(wParam) == BN_CLICKED) {
				ClearSelection();
				m_Group->AddAudio(0);
				Update();
				return true;
			}
			return false;

		case IDC_MOVEAUDIOUP:
			if (HIWORD(wParam) == BN_CLICKED) {
				m_Group->MoveAudioUp(m_Group->GetSelectedAudio());
				Update();
				return true;
			}
			return false;

		case IDC_MOVEAUDIODOWN:
			if (HIWORD(wParam) == BN_CLICKED) {
				m_Group->MoveAudioDown(m_Group->GetSelectedAudio());
				Update();
				return true;
			}
			return false;

		case IDC_DELETEAUDIO:
			if (HIWORD(wParam) == BN_CLICKED) {
				m_Group->DeleteAudio(m_Group->GetSelectedAudio());
				Update();
				return true;
			}
			return false;

		case IDC_AUDIOLANGUAGE:
			switch (HIWORD(wParam)) {
			case CBN_SETFOCUS:
				hasChanged = false;
				return true;
			case CBN_SELCHANGE:
				hasChanged = true;
				return true;
			case CBN_KILLFOCUS:
				if (hasChanged && m_Group->IsAudioSelected()) {
					m_Group->SetAudioLanguage(m_Group->GetSelectedAudio(),
						(int) SendDlgItemMessage(hWnd(), IDC_AUDIOLANGUAGE, CB_GETCURSEL, 0, 0));
					Update();
				}
				hasChanged = false;
				return true;
			}
			return false;

		case IDC_ADDSUBTITLE:
			if (HIWORD(wParam) == BN_CLICKED) {
				ClearSelection();
				m_Group->AddSubtitle(0);
				Update();
				return true;
			}
			return false;

		case IDC_MOVESUBTITLEUP:
			if (HIWORD(wParam) == BN_CLICKED) {
				m_Group->MoveSubtitleUp(m_Group->GetSelectedSubtitle());
				Update();
				return true;
			}
			return false;

		case IDC_MOVESUBTITLEDOWN:
			if (HIWORD(wParam) == BN_CLICKED) {
				m_Group->MoveSubtitleDown(m_Group->GetSelectedSubtitle());
				Update();
				return true;
			}
			return false;

		case IDC_DELETESUBTITLE:
			if (HIWORD(wParam) == BN_CLICKED) {
				m_Group->DeleteSubtitle(m_Group->GetSelectedSubtitle());
				Update();
				return true;
			}
			return false;

		case IDC_SUBTITLELANGUAGE:
			switch (HIWORD(wParam)) {
			case CBN_SETFOCUS:
				hasChanged = false;
				return true;
			case CBN_SELCHANGE:
				hasChanged = true;
				return true;
			case CBN_KILLFOCUS:
				if (hasChanged && m_Group->IsSubtitleSelected()) {
					m_Group->SetSubtitleLanguage(m_Group->GetSelectedSubtitle(),
						(int) SendDlgItemMessage(hWnd(), IDC_SUBTITLELANGUAGE, CB_GETCURSEL, 0, 0));
					Update();
				}
				hasChanged = false;
				return true;
			}
			return false;

		case IDC_FIRSTSUBTITLEON:
			if (HIWORD(wParam) == BN_CLICKED) {
				m_Group->SetFirstSubtitleOn(! m_Group->GetFirstSubtitleOn());
				Update();
				return true;
			}
			return false;

		case IDC_ASPECT_4_3:
			if (HIWORD(wParam) == BN_CLICKED) {
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetVideo()->SetAspectRatio(ASPECTRATIO_4_3);
					}
				}
				Update();
				return true;
			}
			return false;

		case IDC_ASPECT_16_9:
			if (HIWORD(wParam) == BN_CLICKED) {
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetVideo()->SetAspectRatio(ASPECTRATIO_16_9);
					}
				}
				Update();
				return true;
			}
			return false;

		case IDC_ASPECT_2_1:
			if (HIWORD(wParam) == BN_CLICKED) {
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetVideo()->SetAspectRatio(ASPECTRATIO_2_1);
					}
				}
				Update();
				return true;
			}
			return false;

		case IDC_ASPECT_239_1:
			if (HIWORD(wParam) == BN_CLICKED) {
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetVideo()->SetAspectRatio(ASPECTRATIO_239_1);
					}
				}
				Update();
				return true;
			}
			return false;

		case IDC_ASPECT_CUSTOM:
			if (HIWORD(wParam) == BN_CLICKED) {
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetVideo()->SetAspectRatio(ASPECTRATIO_CUSTOM);
					}
				}
				Update();
				return true;
			}
			return false;

		case IDC_ASPECTDEN:
			switch (HIWORD(wParam)) {
			case EN_SETFOCUS:
				hasChanged = false;
				return true;
			case EN_CHANGE:
				hasChanged = true;
				return true;
			case EN_KILLFOCUS:
				if (hasChanged) {
					hasChanged = false;
					int aspectDen = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_ASPECTDEN)).toAnsi().c_str());
					for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
						if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
							(*item)->GetVideo()->SetAspectDen(aspectDen);
						}
					}
					Update();
				}
				return true;
			}
			return false;

		case IDC_ASPECTNOM:
			switch (HIWORD(wParam)) {
			case EN_SETFOCUS:
				hasChanged = false;
				return true;
			case EN_CHANGE:
				hasChanged = true;
				return true;
			case EN_KILLFOCUS:
				if (hasChanged) {
					hasChanged = false;
					int aspectNom = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_ASPECTNOM)).toAnsi().c_str());
					for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
						if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
							(*item)->GetVideo()->SetAspectNom(aspectNom);
						}
					}
					Update();
				}
				return true;
			}
			return false;

		case IDC_COMPENSATEOVERSCAN:
			if (HIWORD(wParam) == BN_CLICKED) {
				bool compensateOverscan = (SendDlgItemMessage(hWnd(), IDC_COMPENSATEOVERSCAN, BM_GETCHECK, 0, 0) != BST_CHECKED);
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetVideo()->SetCompensateOverscan(compensateOverscan);
					}
				}
				Update();
				return true;
			}
			return false;

		case IDC_OVERSCANPERCENT:
			switch (HIWORD(wParam)) {
			case EN_SETFOCUS:
				hasChanged = false;
				return true;
			case EN_CHANGE:
				hasChanged = true;
				return true;
			case EN_KILLFOCUS:
				if (hasChanged) {
					hasChanged = false;
					int overscanPercent = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_OVERSCANPERCENT)).toAnsi().c_str());
					for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
						if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
							(*item)->GetVideo()->SetOverscanPercent(overscanPercent);
						}
					}
					Update();
				}
				return true;
			}
			return false;

		case IDC_CREATECHAPTERS:
			if (HIWORD(wParam) == BN_CLICKED) {
				bool createChapters = (SendDlgItemMessage(hWnd(), IDC_CREATECHAPTERS, BM_GETCHECK, 0, 0) != BST_CHECKED);
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetVideo()->SetCreateChapters(createChapters);
					}
				}
				Update();
				return true;
			}
			return false;

		case IDC_CHAPTERCOUNT:
			switch (HIWORD(wParam)) {
			case EN_SETFOCUS:
				hasChanged = false;
				return true;
			case EN_CHANGE:
				hasChanged = true;
				return true;
			case EN_KILLFOCUS:
				if (hasChanged) {
					hasChanged = false;
					int chapterCount = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_CHAPTERCOUNT)).toAnsi().c_str());
					for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
						if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
							(*item)->GetVideo()->SetChapterCount(chapterCount);
						}
					}
					Update();
				}
				return true;
			}
			return false;

		case IDC_OPENVIDEOUSING:
			switch (HIWORD(wParam)) {
			case CBN_SETFOCUS:
				hasChanged = false;
				return true;
			case CBN_SELCHANGE:
				hasChanged = true;
				return true;
			case CBN_KILLFOCUS:
				if (hasChanged) {
					hasChanged = false;
					String opener;
					int index = SendDlgItemMessage(hWnd(), IDC_OPENVIDEOUSING, CB_GETCURSEL, 0, 0);
					if (index >= 0) {
						int length = (int) SendDlgItemMessage(hWnd(), IDC_OPENVIDEOUSING, CB_GETLBTEXTLEN, index, 0);
						LPTSTR s = new TCHAR[length + 1];
						SendDlgItemMessage(hWnd(), IDC_OPENVIDEOUSING, CB_GETLBTEXT, index, (LPARAM) s);
						opener = s;
						delete [] s;
					}
					AnalyzeDialog analyzeDialog;
					for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
						if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
							if ((*item)->GetVideo()->GetOpener() != opener) {
								(*item)->GetVideo()->SetOpener(opener);
								analyzeDialog.AddVideo((*item)->GetVideo());
							}
						}
					}
					analyzeDialog.DoModal(this);
					Update();
				}
				return true;
			}
			return false;

		case IDC_PREPROCESSVIDEO:
			switch (HIWORD(wParam)) {
			case CBN_SETFOCUS:
				hasChanged = false;
				return true;
			case CBN_SELCHANGE:
				hasChanged = true;
				return true;
			case CBN_KILLFOCUS:
				if (hasChanged) {
					hasChanged = false;
					String preProcessor;
					int index = SendDlgItemMessage(hWnd(), IDC_PREPROCESSVIDEO, CB_GETCURSEL, 0, 0);
					if (index >= 0) {
						int length = (int) SendDlgItemMessage(hWnd(), IDC_PREPROCESSVIDEO, CB_GETLBTEXTLEN, index, 0);
						LPTSTR s = new TCHAR[length + 1];
						SendDlgItemMessage(hWnd(), IDC_PREPROCESSVIDEO, CB_GETLBTEXT, index, (LPARAM) s);
						preProcessor = s;
						delete [] s;
					}
					AnalyzeDialog analyzeDialog;
					for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
						if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
							if ((*item)->GetVideo()->GetPreProcessor() != preProcessor) {
								(*item)->GetVideo()->SetPreProcessor(preProcessor);
								analyzeDialog.AddVideo((*item)->GetVideo());
							}
						}
					}
					analyzeDialog.DoModal(this);
					Update();
				}
				return true;
			}
			return false;

		case IDC_LOOKATVIDEO:
			if (HIWORD(wParam) == BN_CLICKED) {
				Video * video = NULL;
				for (int index = 0; (! video) && (index < (int) m_TitleListItems.size()); index++) {
					if (m_TitleListItems[index]->IsSelected()) {
						if (m_TitleListItems[index]->GetObject() == VIDEO_OBJECT) {
							video = m_TitleListItems[index]->GetVideo();
						}
					}
				}
				if (video) {
					LookAtVideoDialog lavd(this, video->GetFileName(), video->GetOpener(), 
						video->GetUpsideDown(), video->GetPreProcessor(),
						(m_Group->GetProject()->GetDVDFormat() == DVDFORMAT_PAL));
					lavd.DoModal();
				}
				return true;
			}
			return false;

		case IDC_UPSIDEDOWN:
			if (HIWORD(wParam) == BN_CLICKED) {
				bool upsideDown = (SendDlgItemMessage(hWnd(), IDC_UPSIDEDOWN, BM_GETCHECK, 0, 0) != BST_CHECKED);
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetVideo()->SetUpsideDown(upsideDown);
					}
				}
				Update();
				return true;
			}
			return false;

		case IDC_AUTODEINTERLACE:
			if (HIWORD(wParam) == BN_CLICKED) {
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetVideo()->SetInterlacing(INTERLACING_AUTO);
					}
				}
				Update();
				return true;
			}
			return false;

		case IDC_PROGRESSIVE:
			if (HIWORD(wParam) == BN_CLICKED) {
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetVideo()->SetInterlacing(INTERLACING_PROGRESSIVE);
					}
				}
				Update();
				return true;
			}
			return false;

		case IDC_INTERLACED_TFF:
			if (HIWORD(wParam) == BN_CLICKED) {
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetVideo()->SetInterlacing(INTERLACING_TFF);
					}
				}
				Update();
				return true;
			}
			return false;

		case IDC_INTERLACED_BFF:
			if (HIWORD(wParam) == BN_CLICKED) {
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetVideo()->SetInterlacing(INTERLACING_BFF);
					}
				}
				Update();
				return true;
			}
			return false;

		case IDC_DOIVTC:
			if (HIWORD(wParam) == BN_CLICKED) {
				bool doIVTC = (SendDlgItemMessage(hWnd(), IDC_DOIVTC, BM_GETCHECK, 0, 0) != BST_CHECKED);
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetVideo()->SetDoIVTC(doIVTC);
					}
				}
				Update();
				return true;
			}
			return false;

		case IDC_FIXEDCHAPTERS:
			if (HIWORD(wParam) == BN_CLICKED) {
				bool fixedChapters = (SendDlgItemMessage(hWnd(), IDC_FIXEDCHAPTERS, BM_GETCHECK, 0, 0) != BST_CHECKED);
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetVideo()->SetFixedChapters(fixedChapters);
					}
				}
				Update();
				return true;
			}
			return false;
		
		case IDC_CHAPTERSAT:
			switch (HIWORD(wParam)) {
			case EN_SETFOCUS:
				hasChanged = false;
				return true;
			case EN_CHANGE:
				hasChanged = true;
				return true;
			case EN_KILLFOCUS:
				if (hasChanged) {
					hasChanged = false;
					String chaptersAt = String().fromWindow(GetDlgItem(hWnd(), IDC_CHAPTERSAT));
					for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
						if (((*item)->GetObject() == VIDEO_OBJECT) && (*item)->IsSelected()) {
							(*item)->GetVideo()->SetChaptersAt(chaptersAt);
						}
					}
					Update();
				}
				return true;
			}
			return false;

		case IDC_EDITAVSFUNCTIONS:
			if (HIWORD(wParam) == BN_CLICKED) {
				ShellExecute(hWnd(), NULL, g_Config->GetFunctionDir().c_str(), NULL, NULL, SW_SHOWNORMAL);
				return true;
			}
			return false;

		case IDC_AUDIOBITRATE:
			switch (HIWORD(wParam)) {
			case CBN_SETFOCUS:
				hasChanged = false;
				return true;
			case CBN_SELCHANGE:
				hasChanged = true;
				return true;
			case CBN_KILLFOCUS:
				if (hasChanged) {
					hasChanged = false;
					int bitrate = (int) SendDlgItemMessage(hWnd(), IDC_AUDIOBITRATE, CB_GETCURSEL, 0, 0);
					for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
						if (((*item)->GetObject() == AUDIO_OBJECT) && (*item)->IsSelected()) {
							(*item)->GetAudio()->SetBitRate(bitrate);
						}
					}
					Update();
				}
				return true;
			}
			return false;

		case IDC_AUDIOFROMVIDEO:
			if (HIWORD(wParam) == BN_CLICKED) {
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == AUDIO_OBJECT) && ! (*item)->IsTrack() && (*item)->IsSelected()) {
						Audio * audio = (*item)->GetAudio();
						audio->SetFromVideo(true);
						if (! audio->IsChecked()) audio->Analyze();
					}
				}
				Update();
			}
			return false;

		case IDC_AUDIOFROMFILE:
			if (HIWORD(wParam) == BN_CLICKED) {
				AnalyzeDialog analyzeDialog;
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == AUDIO_OBJECT) && ! (*item)->IsTrack() && (*item)->IsSelected()) {
						Audio * audio = (*item)->GetAudio();
						audio->SetFromVideo(false);
						if (! audio->IsChecked()) analyzeDialog.AddAudio(audio);
					}
				}
				if (analyzeDialog.GetCount()) analyzeDialog.DoModal(this);
				Update();
			}
			return false;
		
		case IDC_BROWSEAUDIO:
			if (HIWORD(wParam) == BN_CLICKED) {
				TCHAR fileNames[MAX_PATH * MAX_OPEN_FILES] = TEXT("");
				OpenFileExtData openData;
				openData.opener = g_Config->GetAudioOpener();
				openData.preProcessor = g_Config->GetAudioPreProcessor();
				OPENFILENAME ofn;
				ZeroMemory(&ofn, sizeof(OPENFILENAME));
				ofn.lStructSize     = sizeof(OPENFILENAME);
				ofn.hwndOwner       = hWnd();
				ofn.hInstance       = hInstance;
				ofn.lpstrFilter     = TEXT("Audio files\0")
					TEXT("*.avs;*.aac;*.ac3;*.au;*.aif;*.aiff;*.dts;*.flac;*.iff;*.m4a;*.mp2;*.mp3;*.mpa;*.mpc;*.ogg;*.ra;*.voc;*.wav;*.wma\0")
					TEXT("Video files with audio track\0")
					TEXT("*.avs;*.asf;*.asx;*.avi;*.divx;*.dv;*.flv;*.grf;*.mov;*.mp4;*.mpe;*.mpeg;*.mpg;*.mkv;*.mov;*.ogm;*.qt;*.rm;*.rmvb;*.wmv\0")
					TEXT("All files\0*.*\0\0");
				ofn.lpstrFile       = fileNames;
				ofn.nMaxFile        = MAX_PATH * MAX_OPEN_FILES;
				ofn.lpstrInitialDir = m_Group->GetProject()->GetOpenFileDirectory().c_str();
				ofn.lpstrTitle      = TEXT("Browse audio files");
				ofn.Flags           = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_ALLOWMULTISELECT | OFN_EXPLORER | OFN_ENABLETEMPLATE | OFN_ENABLEHOOK | OFN_ENABLESIZING;
				ofn.lpTemplateName  = MAKEINTRESOURCE(IDD_OPENFILEEXT);
				ofn.lpfnHook        = OpenFileHookProc;
				ofn.lCustData       = (LPARAM) & openData;
				if (GetOpenFileName(& ofn)) {
					int fileCount;
					LPTSTR * fileList = GetOpenFileList(fileNames, & fileCount);
					m_Group->GetProject()->SetOpenFileDirectory(String(fileList[0]).replaceFileName(S("")));
					AnalyzeDialog analyzeDialog;
					int index = 0;
					for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
						if (((*item)->GetObject() == AUDIO_OBJECT) && ! (*item)->IsTrack() && (*item)->IsSelected()) {
							if (index < fileCount) {
								AudioPtr audio = AudioPtr(new Audio((*item)->GetAudio()->GetVideo(), (*item)->GetAudio()));
								audio->SetFromVideo(false);
								audio->SetFileName(fileList[index++], false);
								audio->SetOpener(openData.opener, false);
								audio->SetPreProcessor(openData.preProcessor, false);
								analyzeDialog.AddAudio(audio);
							}
						}
					}
					FreeOpenFileList(fileList);
					ExecuteMessageLoop();
					if (analyzeDialog.DoModal(this)) {
						index = 0;
						for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
							if (((*item)->GetObject() == AUDIO_OBJECT) && ! (*item)->IsTrack() && (*item)->IsSelected()) {
								if (index < fileCount) {
									AudioPtr audio = analyzeDialog.GetAudio(index++);
									(*item)->GetAudio()->GetVideo()->SetAudio((*item)->GetAudio()->GetIndex(), audio);
									(*item)->SetAudio(audio);
									audio->SetChanged();
								}
							}
						}
						Update();
					} else {
						String errorMessage;
						for (int i = 0; i < analyzeDialog.GetCount(); i++) {
							if (! analyzeDialog.GetAudio(i)->IsValid()) {
								errorMessage += analyzeDialog.GetAudio(i)->Recalculate();
							}
						}
						MessageBox(hWnd(), errorMessage.trim().c_str(), APPLICATION, MB_OK | MB_ICONSTOP);
					}
				}
				return true;
			}
			return false;

		case IDC_OPENAUDIOUSING:
			switch (HIWORD(wParam)) {
			case CBN_SETFOCUS:
				hasChanged = false;
				return true;
			case CBN_SELCHANGE:
				hasChanged = true;
				return true;
			case CBN_KILLFOCUS:
				if (hasChanged) {
					hasChanged = false;
					String opener;
					int index = SendDlgItemMessage(hWnd(), IDC_OPENAUDIOUSING, CB_GETCURSEL, 0, 0);
					if (index >= 0) {
						int length = (int) SendDlgItemMessage(hWnd(), IDC_OPENAUDIOUSING, CB_GETLBTEXTLEN, index, 0);
						LPTSTR s = new TCHAR[length + 1];
						SendDlgItemMessage(hWnd(), IDC_OPENAUDIOUSING, CB_GETLBTEXT, index, (LPARAM) s);
						opener = s;
						delete [] s;
					}
					AnalyzeDialog analyzeDialog;
					for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
						if (((*item)->GetObject() == AUDIO_OBJECT) && (*item)->IsSelected()) {
							if ((*item)->GetAudio()->GetOpener() != opener) {
								(*item)->GetAudio()->SetOpener(opener);
								analyzeDialog.AddAudio((*item)->GetAudio());
							}
						}
					}
					analyzeDialog.DoModal(this);
					Update();
				}
				return true;
			}
			return false;

		case IDC_PREPROCESSAUDIO:
			switch (HIWORD(wParam)) {
			case CBN_SETFOCUS:
				hasChanged = false;
				return true;
			case CBN_SELCHANGE:
				hasChanged = true;
				return true;
			case CBN_KILLFOCUS:
				if (hasChanged) {
					hasChanged = false;
					String preProcessor;
					int index = SendDlgItemMessage(hWnd(), IDC_PREPROCESSAUDIO, CB_GETCURSEL, 0, 0);
					if (index >= 0) {
						int length = (int) SendDlgItemMessage(hWnd(), IDC_PREPROCESSAUDIO, CB_GETLBTEXTLEN, index, 0);
						LPTSTR s = new TCHAR[length + 1];
						SendDlgItemMessage(hWnd(), IDC_PREPROCESSAUDIO, CB_GETLBTEXT, index, (LPARAM) s);
						preProcessor = s;
						delete [] s;
					}
					AnalyzeDialog analyzeDialog;
					for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
						if (((*item)->GetObject() == AUDIO_OBJECT) && (*item)->IsSelected()) {
							if ((*item)->GetAudio()->GetPreProcessor() != preProcessor) {
								(*item)->GetAudio()->SetPreProcessor(preProcessor);
								analyzeDialog.AddAudio((*item)->GetAudio());
							}
						}
					}
					analyzeDialog.DoModal(this);
					Update();
				}
				return true;
			}
			return false;

		case IDC_FONTFACE:
			switch (HIWORD(wParam)) {
			case CBN_SETFOCUS:
				hasChanged = false;
				return true;
			case CBN_SELCHANGE:
			case CBN_EDITCHANGE:
				hasChanged = true;
				return true;
			case CBN_KILLFOCUS:
				if (hasChanged) {
					hasChanged = false;
					String fontFace = String().fromWindow(GetDlgItem(hWnd(), IDC_FONTFACE));
					if (GetFontInfo(fontFace.c_str(), NULL, NULL)) {
						for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
							if (((*item)->GetObject() == SUBTITLE_OBJECT) && (*item)->IsSelected()) {
								(*item)->GetSubtitle()->SetFontFace(fontFace);
							}
						}
						Update();
					}
				}
				return true;
			}
			return false;

		case IDC_FONTSIZE:
			switch (HIWORD(wParam)) {
			case EN_SETFOCUS:
				hasChanged = false;
				return true;
			case EN_CHANGE:
				hasChanged = true;
				return true;
			case EN_KILLFOCUS:
				if (hasChanged) {
					hasChanged = false;
					int fontSize = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_FONTSIZE)).toAnsi().c_str());
					for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
						if (((*item)->GetObject() == SUBTITLE_OBJECT) && (*item)->IsSelected()) {
							(*item)->GetSubtitle()->SetFontSize(fontSize);
						}
					}
					Update();
				}
				return true;
			}
			return false;

		case IDC_BOLD:
			if (HIWORD(wParam) == BN_CLICKED) {
				bool bold = (SendDlgItemMessage(hWnd(), IDC_BOLD, BM_GETCHECK, 0, 0) != BST_CHECKED);
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == SUBTITLE_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetSubtitle()->SetBold(bold);
					}
				}
				Update();
			}
			return false;

		case IDC_ITALIC:
			if (HIWORD(wParam) == BN_CLICKED) {
				bool italic = (SendDlgItemMessage(hWnd(), IDC_ITALIC, BM_GETCHECK, 0, 0) != BST_CHECKED);
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == SUBTITLE_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetSubtitle()->SetItalic(italic);
					}
				}
				Update();
			}
			return false;

		case IDC_BOX:
			if (HIWORD(wParam) == BN_CLICKED) {
				bool box = (SendDlgItemMessage(hWnd(), IDC_BOX, BM_GETCHECK, 0, 0) != BST_CHECKED);
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == SUBTITLE_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetSubtitle()->SetBox(box);
					}
				}
				Update();
			}
			return false;
		
		case IDC_CHOOSECOLOR:
			if (HIWORD(wParam) == BN_CLICKED) {
				CHOOSECOLOR cc;
				cc.lStructSize    = sizeof(CHOOSECOLOR);
				cc.hwndOwner      = hWnd();
				cc.hInstance      = NULL;
				cc.lpCustColors   = CustomColors;
				cc.Flags          = CC_ANYCOLOR | CC_FULLOPEN | CC_RGBINIT;
				cc.lCustData      = 0;
				cc.lpfnHook       = NULL;
				cc.lpTemplateName = NULL;
				cc.rgbResult      = GetWindowLong(GetDlgItem(hWnd(), IDC_VIEWCOLOR), GWL_USERDATA);
				if (ChooseColor(&cc)) {
					for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
						if (((*item)->GetObject() == SUBTITLE_OBJECT) && (*item)->IsSelected()) {
							(*item)->GetSubtitle()->SetColor(cc.rgbResult);
						}
					}
					Update();
				}
				return true;
			}
			return false;

		case IDC_BROWSESUBTITLE:
			if (HIWORD(wParam) == BN_CLICKED) {
				TCHAR fileNames[MAX_PATH * MAX_OPEN_FILES] = TEXT("");
				// GetDlgItemText(hWnd(), IDC_SUBTITLEFILE, fileNames, MAX_PATH * MAX_OPEN_FILES);
				OPENFILENAME ofn;
				ZeroMemory(&ofn, sizeof(OPENFILENAME));
				ofn.lStructSize     = sizeof(OPENFILENAME);
				ofn.hwndOwner       = hWnd();
				ofn.hInstance       = hInstance;
				ofn.lpstrFilter     = TEXT("Subtitle files\0")
					TEXT("*.ass;*.ssa;*.srt;*.sub\0")
					TEXT("All files\0*.*\0\0");
				ofn.lpstrFile       = fileNames;
				ofn.nMaxFile        = MAX_PATH * MAX_OPEN_FILES;
				ofn.lpstrInitialDir = m_Group->GetProject()->GetOpenFileDirectory().c_str();
				ofn.lpstrTitle      = TEXT("Browse subtitle files");
				ofn.Flags           = OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_ALLOWMULTISELECT | OFN_EXPLORER;
				if (GetOpenFileName(& ofn)) {
					int fileCount;
					LPTSTR * fileList = GetOpenFileList(fileNames, & fileCount);
					m_Group->GetProject()->SetOpenFileDirectory(String(fileList[0]).replaceFileName(S("")));
					AnalyzeDialog analyzeDialog;
					int index = 0;
					for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
						if (((*item)->GetObject() == SUBTITLE_OBJECT) && ! (*item)->IsTrack() && (*item)->IsSelected()) {
							if (index < fileCount) {
								SubtitlePtr subtitle = SubtitlePtr(new Subtitle((*item)->GetSubtitle()->GetVideo(), (*item)->GetSubtitle()));
								subtitle->SetFileName(fileList[index++], false);
								analyzeDialog.AddSubtitle(subtitle);
							}
						}
					}
					FreeOpenFileList(fileList);
					ExecuteMessageLoop();
					if (analyzeDialog.DoModal(this)) {
						index = 0;
						for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
							if (((*item)->GetObject() == SUBTITLE_OBJECT) && ! (*item)->IsTrack() && (*item)->IsSelected()) {
								if (index < fileCount) {
									SubtitlePtr subtitle = analyzeDialog.GetSubtitle(index++);
									(*item)->GetSubtitle()->GetVideo()->SetSubtitle((*item)->GetSubtitle()->GetIndex(), subtitle);
									(*item)->SetSubtitle(subtitle);
									subtitle->SetChanged();
								}
							}
						}
						Update();
					} else {
						String errorMessage;
						for (int i = 0; i < analyzeDialog.GetCount(); i++) {
							if (! analyzeDialog.GetSubtitle(i)->IsValid()) {
								errorMessage += analyzeDialog.GetSubtitle(i)->Recalculate();
							}
						}
						MessageBox(hWnd(), errorMessage.trim().c_str(), APPLICATION, MB_OK | MB_ICONSTOP);
					}
				}
				return true;
			}
			return false;

		case IDC_CHARSET:
			switch (HIWORD(wParam)) {
			case CBN_SETFOCUS:
				hasChanged = false;
				return true;
			case CBN_SELCHANGE:
				hasChanged = true;
				return true;
			case CBN_KILLFOCUS:
				if (hasChanged) {
					hasChanged = false;
					int charset = (int) SendDlgItemMessage(hWnd(), IDC_CHARSET, CB_GETCURSEL, 0, 0);
					for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
						if (((*item)->GetObject() == SUBTITLE_OBJECT) && (*item)->IsSelected()) {
							(*item)->GetSubtitle()->SetCharset(charset);
						}
					}
					Update();
				}
				return true;
			}
			return false;

		case IDC_DELAYSUBTITLE:
			if (HIWORD(wParam) == BN_CLICKED) {
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == SUBTITLE_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetSubtitle()->SetAdvance(false);
					}
				}
				Update();
			}
			return false;

		case IDC_ADVANCESUBTITLE:
			if (HIWORD(wParam) == BN_CLICKED) {
				for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
					if (((*item)->GetObject() == SUBTITLE_OBJECT) && (*item)->IsSelected()) {
						(*item)->GetSubtitle()->SetAdvance(true);
					}
				}
				Update();
			}
			return false;

		case IDC_SUBTITLEOFFSET:
			switch (HIWORD(wParam)) {
			case EN_SETFOCUS:
				hasChanged = false;
				return true;
			case EN_CHANGE:
				hasChanged = true;
				return true;
			case EN_KILLFOCUS:
				if (hasChanged) {
					hasChanged = false;
					int offset = atoi(String().fromWindow(GetDlgItem(hWnd(), IDC_SUBTITLEOFFSET)).toAnsi().c_str());
					for (TitleListItemVector::iterator item = m_TitleListItems.begin(); item != m_TitleListItems.end(); item++) {
						if (((*item)->GetObject() == SUBTITLE_OBJECT) && (*item)->IsSelected()) {
							(*item)->GetSubtitle()->SetOffset(offset);
						}
					}
					Update();
				}
				return true;
			}
			return false;

		case IDC_BACK:
			if (HIWORD(wParam) == BN_CLICKED) {
				SendMessage(hWnd(), WM_CLOSE, 0, 0);
				return true;
			}
			return false;

		default:
			return false;
		}

	case WM_MEASUREITEM:
		MeasureItem((LPMEASUREITEMSTRUCT) lParam);
		return true;

	case WM_DRAWITEM:
		DrawItem((LPDRAWITEMSTRUCT) lParam);
		return true;

	case WM_CLOSE:
		EndDialog(hWnd(), 0);
		return true;
	}
	return false;
}
