#ifndef VIDEO_H_
#define VIDEO_H_

#include "managableptr.h"
#include "managedptr.h"
#include "string.h"

#include <vector>

#include "audio.h"
#include "subtitle.h"

#define ASPECTRATIO_4_3    0
#define ASPECTRATIO_16_9   1
#define ASPECTRATIO_2_1    2
#define ASPECTRATIO_239_1  3
#define ASPECTRATIO_CUSTOM 4
#define ASPECTRATIO_MAX    5

#define INTERLACING_AUTO        0
#define INTERLACING_PROGRESSIVE 1
#define INTERLACING_TFF         2
#define INTERLACING_BFF         3
#define INTERLACING_MAX         4

class Group;
class Title;

class Video : public ManagablePtr
{
public:
	Video(Group * group);
	virtual ~Video();

public:
	Group * GetGroup() const;
	void AddTracks(bool selectThem = false);
	void SetTitle(Title * title);
	Title * GetTitle() const;
	void SetTitleNo(int titleNo);
	int GetTitleNo() const;
	void SetPartNo(int partNo);
	int GetPartNo() const;
	bool IsFileSelected() const;
	void SetFileSelected(bool selected);
	bool IsTrackSelected() const;
	void SetTrackSelected(bool selected);
	bool IsTitleSelected() const;
	void SetTitleSelected(bool selected);
	bool IsSelected() const;
	
	const String & GetFileName() const;
	bool SetFileName(const String & fileName, bool setChanged = true);
	bool IsMergedWithNext() const;
	bool SetMergeWithNext(bool mergeWithNext, bool setChanged = true);
	bool IsMergedWithPrev() const;
	void SetMergeWithPrev(bool mergeWithPrev);
	int  GetAspectRatio() const;
	bool SetAspectRatio(int aspectRatio, bool setChanged = true);
	int  GetAspectNom() const;
	bool SetAspectNom(int aspectNom, bool setChanged = true);
	int  GetAspectDen() const;
	bool SetAspectDen(int aspectDen, bool setChanged = true);
	double GetOrigAspectRatio() const;
	String GetOrigAspectRatioStr() const;
	double GetFinalAspectRatio() const;
	String GetFinalAspectRatioStr() const;
	bool GetCompensateOverscan() const;
	bool SetCompensateOverscan(bool compensateOverscan, bool setChanged = true);
	int GetOverscanPercent() const;
	bool SetOverscanPercent(int overscanPercent, bool setChanged = true);
	bool GetCreateChapters() const;
	bool SetCreateChapters(bool createChapters, bool setChanged = true);
	int  GetChapterCount() const;
	bool SetChapterCount(int chapterCount, bool setChanged = true);
	String GetChaptersAt() const;
	bool SetChaptersAt(const String & chaptersAt, bool setChanged = true);
	const std::vector<double> & GetChapterVector() const;
	bool GetUpsideDown() const;
	bool SetUpsideDown(bool upsideDown, bool setChanged = true);
	int  GetInterlacing() const;
	bool SetInterlacing(int interlacing, bool setChanged = true);
	bool GetDoIVTC() const;
	bool SetDoIVTC(bool doIVTC, bool setChanged = true);
	bool GetFixedChapters() const;
	bool SetFixedChapters(bool fixedChapters, bool setChanged = true);
	String GetOpener() const;
	bool SetOpener(const String & opener, bool setChanged = true);
	const String & GetPreProcessor() const;
	bool SetPreProcessor(const String & preProcessor, bool setChanged = true);
	void SetChanged();

	int GetAudioCount() const;
	AudioPtr GetAudio(int index) const;
	void SetAudio(int index, AudioPtr audio);
	void AddAudio(bool selected = false);
	void MoveAudioUp(int index);
	void MoveAudioDown(int index);
	void DeleteAudio(int index);

	int GetSubtitleCount() const;
	SubtitlePtr GetSubtitle(int index) const;
	void SetSubtitle(int index, SubtitlePtr subtitle);
	void AddSubtitle(bool selected = false);
	void MoveSubtitleUp(int index);
	void MoveSubtitleDown(int index);
	void DeleteSubtitle(int index);

	void SetInvalid(bool clearChecked);
	bool IsChecked() const;
	bool Analyze();
	bool IsScript() const;
	const AVSWrapper::Info & GetInfo() const;
	bool IsValid() const;
	int GetWidth() const;
	int GetHeight() const;
	int GetFrameCount() const;
	double GetOrigFPS() const;
	bool DoConvertFPS() const;
	double ConvertToFPS() const;
	bool AssumeFPS() const;
	double GetOrigLength() const;
	double GetLength() const;
	double GetAcceleration() const;
	bool IsInterlaced() const;
	bool IsBFF() const;
	bool HasAudio() const;
	int GetNoOfChannels() const;

	int GetIndex() const;
	void SetIndex(int index);
	String GetIdent(bool forTrack);
	String Recalculate();
	int GetSubStreamRate() const;
	void GetDescription(StringVectorVector & strings, bool forTitle) const;
	bool Load(csv_file_t csv, int & lineno, int nest);
	void Save(FILE * f, int nest) const;

protected:
	Group * m_Group;
	String m_FileName;
	bool   m_MergeWithNext;
	bool   m_MergeWithPrev;
	int    m_AspectRatio;
	int    m_AspectDen;
	int    m_AspectNom;
	bool   m_CompensateOverscan;
	int    m_OverscanPercent;
	bool   m_UpsideDown;
	bool   m_CreateChapters;
	int    m_ChapterCount;
	int    m_Interlacing;
	bool   m_DoIVTC;
	bool   m_FixedChapters;
	std::vector<double> m_ChaptersAt;
	String m_Opener;
	String m_PreProcessor;
	AudioVector m_Audios;
	SubtitleVector m_Subtitles;
	Title * m_Title;
	int    m_TitleNo;
	int    m_PartNo;
	bool   m_IsChecked;
	AVSWrapper::Info m_Info;
	String m_OpenError;
	bool   m_DoConvertFPS;
	double m_ConvertToFPS;
	bool   m_AssumeFPS;
	double m_Length;
	double m_Acceleration;
	int    m_SubStreamRate;
	bool   m_FileSelected;
	bool   m_TrackSelected;
	bool   m_TitleSelected;
	int    m_Index;
};

typedef ManagedPtr<Video> VideoPtr;
typedef std::vector<VideoPtr> VideoVector;

#endif // VIDEO_H_
